@@ -12,6 +12,7 @@ import sys
 import time
 import webbrowser
 from datetime import datetime
+from functools import partial
 from pathlib import Path
 from queue import Queue
 from typing import List, Literal
@@ -41,6 +42,54 @@ from murfey.util import _get_visit_list
 log = logging.getLogger("murfey.client")
 
 
+def read_config() -> configparser.ConfigParser:
+    config = configparser.ConfigParser()
+    try:
+        mcch = os.environ.get("MURFEY_CLIENT_CONFIG_HOME")
+        murfey_client_config_home = Path(mcch) if mcch else Path.home()
+        with open(murfey_client_config_home / ".murfey") as configfile:
+            config.read_file(configfile)
+    except FileNotFoundError:
+        log.warning(
+            f"Murfey client configuration file {murfey_client_config_home / '.murfey'} not found"
+        )
+    if "Murfey" not in config:
+        config["Murfey"] = {}
+    return config
+
+
+token = read_config()["Murfey"].get("token", "")
+
+requests.get = partial(requests.get, headers={"Authorization": f"Bearer {token}"})
+requests.post = partial(requests.post, headers={"Authorization": f"Bearer {token}"})
+requests.delete = partial(requests.delete, headers={"Authorization": f"Bearer {token}"})
+
+
+def write_config(config: configparser.ConfigParser):
+    mcch = os.environ.get("MURFEY_CLIENT_CONFIG_HOME")
+    murfey_client_config_home = Path(mcch) if mcch else Path.home()
+    with open(murfey_client_config_home / ".murfey", "w") as configfile:
+        config.write(configfile)
+
+
+def main_loop(
+    source_watchers: List[murfey.client.watchdir.DirWatcher],
+    appearance_time: float,
+    transfer_all: bool,
+):
+    log.info(
+        f"Murfey {murfey.__version__} on Python {'.'.join(map(str, sys.version_info[0:3]))} entering main loop"
+    )
+    if appearance_time > 0:
+        modification_time: float | None = time.time() - appearance_time * 3600
+    else:
+        modification_time = None
+    while True:
+        for sw in source_watchers:
+            sw.scan(modification_time=modification_time, transfer_all=transfer_all)
+        time.sleep(15)
+
+
 def _enable_webbrowser_in_cygwin():
     """Helper function to make webbrowser.open() work in CygWin"""
     if "cygwin" in platform.system().lower() and shutil.which("cygstart"):
@@ -76,6 +125,7 @@ def _check_for_updates(
 
 def run():
     config = read_config()
+    instrument_name = config["Murfey"]["instrument_name"]
     try:
         server_routing = config["ServerRouter"]
     except KeyError:
@@ -165,12 +215,6 @@ def run():
         default=False,
         help="Remove source files immediately after their transfer",
     )
-    parser.add_argument(
-        "--relax",
-        action="store_true",
-        default=False,
-        help="Relax the condition that the source directory needs to be recognised from the configuration",
-    )
     parser.add_argument(
         "--name",
         type=str,
@@ -233,7 +277,7 @@ def run():
                 break
     if not ongoing_visits:
         print("Ongoing visits:")
-        ongoing_visits = _get_visit_list(murfey_url)
+        ongoing_visits = _get_visit_list(murfey_url, instrument_name)
         pprint(ongoing_visits)
         ongoing_visits = [v.name for v in ongoing_visits]
 
@@ -263,12 +307,15 @@ def run():
 
     status_bar = StatusBar()
 
-    machine_data = requests.get(f"{murfey_url.geturl()}/machine/").json()
+    machine_data = requests.get(
+        f"{murfey_url.geturl()}/instruments/{instrument_name}/machine"
+    ).json()
     gain_ref: Path | None = None
 
     instance_environment = MurfeyInstanceEnvironment(
         url=murfey_url,
         client_id=ws.id,
+        instrument_name=instrument_name,
         software_versions=machine_data.get("software_versions", {}),
         # sources=[Path(args.source)],
         # watchers=source_watchers,
@@ -291,48 +338,8 @@ def run():
         gain_ref=gain_ref,
         redirected_logger=rich_handler,
         force_mdoc_metadata=not args.ignore_mdoc_metadata,
-        strict=not args.relax,
         processing_enabled=machine_data.get("processing_enabled", True),
         skip_existing_processing=args.skip_existing_processing,
     )
     app.run()
     rich_handler.redirect = False
-
-
-def main_loop(
-    source_watchers: List[murfey.client.watchdir.DirWatcher],
-    appearance_time: float,
-    transfer_all: bool,
-):
-    log.info(
-        f"Murfey {murfey.__version__} on Python {'.'.join(map(str, sys.version_info[0:3]))} entering main loop"
-    )
-    if appearance_time > 0:
-        modification_time: float | None = time.time() - appearance_time * 3600
-    else:
-        modification_time = None
-    while True:
-        for sw in source_watchers:
-            sw.scan(modification_time=modification_time, transfer_all=transfer_all)
-        time.sleep(15)
-
-
-def read_config() -> configparser.ConfigParser:
-    config = configparser.ConfigParser()
-    try:
-        mcch = os.environ.get("MURFEY_CLIENT_CONFIG_HOME")
-        murfey_client_config_home = Path(mcch) if mcch else Path.home()
-        with open(murfey_client_config_home / ".murfey") as configfile:
-            config.read_file(configfile)
-    except FileNotFoundError:
-        pass
-    if "Murfey" not in config:
-        config["Murfey"] = {}
-    return config
-
-
-def write_config(config: configparser.ConfigParser):
-    mcch = os.environ.get("MURFEY_CLIENT_CONFIG_HOME")
-    murfey_client_config_home = Path(mcch) if mcch else Path.home()
-    with open(murfey_client_config_home / ".murfey", "w") as configfile:
-        config.write(configfile)
