@@ -15,10 +15,17 @@ from murfey.client.instance_environment import (
     MurfeyID,
     MurfeyInstanceEnvironment,
 )
-from murfey.util import capture_get, capture_post, get_machine_config
+from murfey.util import (
+    authorised_requests,
+    capture_get,
+    capture_post,
+    get_machine_config_client,
+)
 
 logger = logging.getLogger("murfey.client.contexts.spa")
 
+requests.get, requests.post, requests.put, requests.delete = authorised_requests()
+
 
 class FoilHole(NamedTuple):
     session_id: int
@@ -34,6 +41,7 @@ class FoilHole(NamedTuple):
     thumbnail_size_y: Optional[int] = None
     pixel_size: Optional[float] = None
     image: str = ""
+    diameter: Optional[float] = None
 
 
 class GridSquare(NamedTuple):
@@ -52,9 +60,18 @@ class GridSquare(NamedTuple):
     tag: str = ""
 
 
-def _get_grid_square_atlas_positions(
-    xml_path: Path, grid_square: str = ""
-) -> Dict[str, Tuple[Optional[int], Optional[int], Optional[float], Optional[float]]]:
+def _get_grid_square_atlas_positions(xml_path: Path, grid_square: str = "") -> Dict[
+    str,
+    Tuple[
+        Optional[int],
+        Optional[int],
+        Optional[float],
+        Optional[float],
+        Optional[int],
+        Optional[int],
+        Optional[float],
+    ],
+]:
     with open(
         xml_path,
         "r",
@@ -64,7 +81,16 @@ def _get_grid_square_atlas_positions(
         "TileXml"
     ]
     gs_pix_positions: Dict[
-        str, Tuple[Optional[int], Optional[int], Optional[float], Optional[float]]
+        str,
+        Tuple[
+            Optional[int],
+            Optional[int],
+            Optional[float],
+            Optional[float],
+            Optional[int],
+            Optional[int],
+            Optional[float],
+        ],
     ] = {}
     for ti in tile_info:
         try:
@@ -89,6 +115,13 @@ def _get_grid_square_atlas_positions(
                     * 1e9,
                     float(gs["value"]["b:PositionOnTheAtlas"]["c:Physical"]["d:y"])
                     * 1e9,
+                    int(
+                        float(gs["value"]["b:PositionOnTheAtlas"]["c:Size"]["d:width"])
+                    ),
+                    int(
+                        float(gs["value"]["b:PositionOnTheAtlas"]["c:Size"]["d:height"])
+                    ),
+                    float(gs["value"]["b:PositionOnTheAtlas"]["c:Rotation"]),
                 )
                 if grid_square:
                     break
@@ -98,8 +131,10 @@ def _get_grid_square_atlas_positions(
 def _file_transferred_to(
     environment: MurfeyInstanceEnvironment, source: Path, file_path: Path
 ):
-    machine_config = get_machine_config(
-        str(environment.url.geturl()), demo=environment.demo
+    machine_config = get_machine_config_client(
+        str(environment.url.geturl()),
+        instrument_name=environment.instrument_name,
+        demo=environment.demo,
     )
     if environment.visit in environment.default_destinations[source]:
         return (
@@ -127,9 +162,9 @@ def _foil_hole_from_file(f: Path) -> int:
 
 
 def _grid_square_metadata_file(
-    f: Path, data_directories: Dict[Path, str], visit: str, grid_square: int
+    f: Path, data_directories: List[Path], visit: str, grid_square: int
 ) -> Path:
-    for dd in data_directories.keys():
+    for dd in data_directories:
         if str(f).startswith(str(dd)):
             base_dir = dd
             mid_dir = f.relative_to(dd).parent
@@ -153,7 +188,7 @@ def _grid_square_data(xml_path: Path, grid_square: int, session_id: int) -> Grid
     )
     if image_paths:
         image_paths.sort(key=lambda x: x.stat().st_ctime)
-        image_path = image_paths[0]
+        image_path = image_paths[-1]
         with open(Path(image_path).with_suffix(".xml")) as gs_xml:
             gs_xml_data = xmltodict.parse(gs_xml.read())
         readout_area = gs_xml_data["MicroscopeImage"]["microscopeData"]["acquisition"][
@@ -168,8 +203,8 @@ def _grid_square_data(xml_path: Path, grid_square: int, session_id: int) -> Grid
             session_id=session_id,
             readout_area_x=full_size[0] if image_path else None,
             readout_area_y=full_size[1] if image_path else None,
-            thumbnail_size_x=None,
-            thumbnail_size_y=None,
+            thumbnail_size_x=int((512 / max(full_size)) * full_size[0]),
+            thumbnail_size_y=int((512 / max(full_size)) * full_size[1]),
             pixel_size=float(pixel_size) if image_path else None,
             image=str(image_path),
         )
@@ -212,6 +247,7 @@ def _foil_hole_data(
         for fh_block in serialization_array[required_key]:
             pix = fh_block["b:value"]["PixelCenter"]
             stage = fh_block["b:value"]["StagePosition"]
+            diameter = fh_block["b:value"]["PixelWidthHeight"]["c:width"]
             if int(fh_block["b:key"]) == foil_hole:
                 return FoilHole(
                     id=foil_hole,
@@ -227,6 +263,7 @@ def _foil_hole_data(
                     thumbnail_size_y=None,
                     pixel_size=float(pixel_size) if image_path else None,
                     image=str(image_path),
+                    diameter=diameter,
                 )
     logger.warning(
         f"Foil hole positions could not be determined from metadata file {xml_path} for foil hole {foil_hole}"
@@ -410,7 +447,7 @@ class _SPAContext(Context):
         binning_factor = 1
         if environment:
             server_config_response = capture_get(
-                f"{str(environment.url.geturl())}/machine/", catch=True
+                f"{str(environment.url.geturl())}/instruments/{environment.instrument_name}/machine"
             )
             if server_config_response is None:
                 return None
@@ -533,7 +570,7 @@ class SPAModularContext(_SPAContext):
         grid_square = _grid_square_from_file(transferred_file)
         grid_square_metadata_file = _grid_square_metadata_file(
             transferred_file,
-            {Path(d): l for d, l in machine_config["data_directories"].items()},
+            [Path(p) for p in machine_config["data_directories"]],
             environment.visit,
             grid_square,
         )
@@ -548,7 +585,10 @@ class SPAModularContext(_SPAContext):
                 Optional[int],
                 Optional[float],
                 Optional[float],
-            ] = (None, None, None, None)
+                Optional[int],
+                Optional[int],
+                Optional[float],
+            ] = (None, None, None, None, None, None, None)
             data_collection_group = (
                 requests.get(
                     f"{str(environment.url.geturl())}/sessions/{environment.murfey_session}/data_collection_groups"
@@ -602,6 +642,9 @@ class SPAModularContext(_SPAContext):
                     "y_location": gs_pix_position[1],
                     "x_stage_position": gs_pix_position[2],
                     "y_stage_position": gs_pix_position[3],
+                    "width": gs_pix_position[4],
+                    "height": gs_pix_position[5],
+                    "angle": gs_pix_position[6],
                 },
             )
         foil_hole = _foil_hole_from_file(transferred_file)
@@ -642,6 +685,7 @@ class SPAModularContext(_SPAContext):
                         "thumbnail_size_x": fh.thumbnail_size_x,
                         "thumbnail_size_y": fh.thumbnail_size_y,
                         "pixel_size": fh.pixel_size,
+                        "diameter": fh.diameter,
                         "tag": str(source),
                         "image": str(image_path),
                     },
@@ -660,17 +704,23 @@ class SPAModularContext(_SPAContext):
     def post_transfer(
         self,
         transferred_file: Path,
-        role: str = "",
         environment: MurfeyInstanceEnvironment | None = None,
         **kwargs,
     ) -> bool:
+        super().post_transfer(
+            transferred_file=transferred_file,
+            environment=environment,
+            **kwargs,
+        )
         data_suffixes = (".mrc", ".tiff", ".tif", ".eer")
-        if role == "detector" and "gain" not in transferred_file.name:
+        if "gain" not in transferred_file.name:
             if transferred_file.suffix in data_suffixes:
                 if self._acquisition_software == "epu":
                     if environment:
-                        machine_config = get_machine_config(
-                            str(environment.url.geturl()), demo=environment.demo
+                        machine_config = get_machine_config_client(
+                            str(environment.url.geturl()),
+                            instrument_name=environment.instrument_name,
+                            demo=environment.demo,
                         )
                     else:
                         machine_config = {}
@@ -700,7 +750,6 @@ class SPAModularContext(_SPAContext):
                         if not environment.movie_counters.get(str(source)):
                             movie_counts_get = capture_get(
                                 f"{str(environment.url.geturl())}/num_movies",
-                                catch=True,
                             )
                             if movie_counts_get is not None:
                                 environment.movie_counters[str(source)] = count(
@@ -714,7 +763,7 @@ class SPAModularContext(_SPAContext):
                         eer_fractionation_file = None
                         if file_transferred_to.suffix == ".eer":
                             response = capture_post(
-                                f"{str(environment.url.geturl())}/visits/{environment.visit}/eer_fractionation_file",
+                                f"{str(environment.url.geturl())}/visits/{environment.visit}/{environment.murfey_session}/eer_fractionation_file",
                                 json={
                                     "eer_path": str(file_transferred_to),
                                     "fractionation": environment.data_collection_parameters[
@@ -725,7 +774,6 @@ class SPAModularContext(_SPAContext):
                                     ],
                                     "fractionation_file_name": "eer_fractionation_spa.txt",
                                 },
-                                catch=True,
                             )
                             if response is None:
                                 return False
@@ -741,7 +789,7 @@ class SPAModularContext(_SPAContext):
                             # try to continue if position information gathering fails so that movie is processed anyway
                             foil_hole = None
 
-                        preproc_url = f"{str(environment.url.geturl())}/visits/{environment.visit}/{environment.client_id}/spa_preprocess"
+                        preproc_url = f"{str(environment.url.geturl())}/visits/{environment.visit}/{environment.murfey_session}/spa_preprocess"
                         preproc_data = {
                             "path": str(file_transferred_to),
                             "description": "",
@@ -764,7 +812,7 @@ class SPAModularContext(_SPAContext):
                                 "gain_ref"
                             ),
                             "extract_downscale": environment.data_collection_parameters.get(
-                                "downscale"
+                                "downscale", True
                             ),
                             "eer_fractionation_file": eer_fractionation_file,
                             "tag": str(source),
@@ -776,7 +824,6 @@ class SPAModularContext(_SPAContext):
                                 k: None if v == "None" else v
                                 for k, v in preproc_data.items()
                             },
-                            catch=True,
                         )
 
         return True
@@ -837,12 +884,11 @@ class SPAContext(_SPAContext):
             "slit_width": data.get("slit_width"),
             "phase_plate": data.get("phase_plate", False),
         }
-        capture_post(url, json=json, catch=True)
+        capture_post(url, json=json)
 
     def post_transfer(
         self,
         transferred_file: Path,
-        role: str = "",
         environment: MurfeyInstanceEnvironment | None = None,
         **kwargs,
     ) -> bool:
@@ -857,9 +903,11 @@ class SPAContext(_SPAContext):
         logger.info(f"registering processing job with parameters: {parameters}")
         parameters = parameters or {}
         environment.id_tag_registry["processing_job"].append(tag)
-        proc_url = f"{str(environment.url.geturl())}/visits/{environment.visit}/{environment.client_id}/register_processing_job"
-        machine_config = get_machine_config(
-            str(environment.url.geturl()), demo=environment.demo
+        proc_url = f"{str(environment.url.geturl())}/visits/{environment.visit}/{environment.murfey_session}/register_processing_job"
+        machine_config = get_machine_config_client(
+            str(environment.url.geturl()),
+            instrument_name=environment.instrument_name,
+            demo=environment.demo,
         )
         image_directory = str(
             Path(machine_config.get("rsync_basepath", "."))
@@ -873,6 +921,7 @@ class SPAContext(_SPAContext):
             )
         msg: Dict[str, Any] = {
             "tag": tag,
+            "source": tag,
             "recipe": "ispyb-relion",
             "parameters": {
                 "acquisition_software": parameters["acquisition_software"],
