@@ -34,6 +34,7 @@ global_env_lock = RLock()
 class MurfeyInstanceEnvironment(BaseModel):
     url: ParseResult
     client_id: int
+    instrument_name: str
     software_versions: Dict[str, str] = {}
     sources: List[Path] = []
     default_destinations: Dict[Path, str] = {}
@@ -52,7 +53,6 @@ class MurfeyInstanceEnvironment(BaseModel):
     }
     data_collection_parameters: dict = {}
     movies: Dict[Path, MovieTracker] = {}
-    motion_corrected_movies: Dict[Path, List[str]] = {}
     listeners: Dict[str, Set[Callable]] = {}
     movie_tilt_pair: Dict[Path, str] = {}
     tilt_angles: Dict[str, List[List[str]]] = {}
@@ -117,7 +117,6 @@ class MurfeyInstanceEnvironment(BaseModel):
         self.autoproc_program_ids = {}
         self.data_collection_parameters = {}
         self.movies = {}
-        self.motion_corrected_movies = {}
         self.listeners = {}
         self.movie_tilt_pair = {}
         self.tilt_angles = {}
