@@ -1,17 +1,23 @@
+"""
+rsync is a Linux process that does fast, versatile remote (and local) file copying.
+This module allows Murfey to use rsync to facilitate its file transfers, and to
+parse the logs outputted by rsync in order confirm that the file has been correctly
+transferred.
+"""
+
 from __future__ import annotations
 
 import logging
 import os
 import queue
+import subprocess
 import threading
 import time
 from enum import Enum
 from pathlib import Path
-from typing import List, NamedTuple
+from typing import Callable, List, NamedTuple
 from urllib.parse import ParseResult
 
-import procrunner
-
 from murfey.client.tui.status_bar import StatusBar
 from murfey.util import Observer
 
@@ -49,6 +55,7 @@ class RSyncer(Observer):
         basepath_local: Path,
         basepath_remote: Path,
         server_url: ParseResult,
+        stop_callback: Callable = lambda *args, **kwargs: None,
         local: bool = False,
         status_bar: StatusBar | None = None,
         do_transfer: bool = True,
@@ -62,13 +69,14 @@ class RSyncer(Observer):
         self._do_transfer = do_transfer
         self._remove_files = remove_files
         self._required_substrings_for_removal = required_substrings_for_removal
+        self._stop_callback = stop_callback
         self._local = local
         self._server_url = server_url
         self._notify = notify
         if local:
             self._remote = str(basepath_remote)
         else:
-            self._remote = f"{server_url.hostname}::{basepath_remote}"
+            self._remote = f"{server_url.hostname}::{basepath_remote}/"
         # For local tests you can use something along the lines of
         # self._remote = f"wra62962@ws133:/dls/tmp/wra62962/junk/{basepath_remote}"
         # to avoid having to set up an rsync daemon
@@ -141,6 +149,16 @@ class RSyncer(Observer):
         logger.info(f"RSync thread starting for {self}")
         self.thread.start()
 
+    def restart(self):
+        self.thread.join()
+        self._halt_thread = False
+        self.thread = threading.Thread(
+            name=f"RSync {self._basepath}:{self._remote}",
+            target=self._process,
+            daemon=True,
+        )
+        self.start()
+
     def stop(self):
         logger.debug("RSync thread stop requested")
         self._stopping = True
@@ -154,6 +172,19 @@ class RSyncer(Observer):
             self.thread.join()
         logger.debug("RSync thread stop completed")
 
+    def finalise(self):
+        self.stop()
+        self._remove_files = True
+        self._notify = False
+        self.thread = threading.Thread(
+            name=f"RSync finalisation {self._basepath}:{self._remote}",
+            target=self._process,
+            daemon=True,
+        )
+        for f in self._basepath.glob("**/*"):
+            self.queue.put(f)
+        self.stop()
+
     def enqueue(self, file_path: Path):
         if not self._stopping:
             absolute_path = (self._basepath / file_path).resolve()
@@ -217,6 +248,7 @@ class RSyncer(Observer):
                 self.queue.task_done()
                 continue
 
+        self._stop_callback(self._basepath, explicit_stop=self._stopping)
         logger.info("RSync thread finished")
 
     def _fake_transfer(self, files: list[Path]) -> bool:
@@ -250,22 +282,39 @@ class RSyncer(Observer):
 
     def _transfer(self, files: list[Path]) -> bool:
         """
-        Actually transfer files in an rsync subprocess
+        Transfer files via an rsync sub-process, and parses the rsync stdout to verify
+        the success of the transfer.
         """
-        previously_transferred = self._files_transferred
 
-        next_file: RSyncerUpdate | None = None
+        # Set up initial variables
+        files = [f for f in files if f.is_file()]
+        previously_transferred = self._files_transferred
         transfer_success: set[Path] = set()
         successful_updates: list[RSyncerUpdate] = []
-
-        files = [f for f in files if f.is_file()]
+        next_file: RSyncerUpdate | None = None
 
         def parse_stdout(line: str):
+            """
+            Reads the stdout from rsync in order to verify the status of transferred
+            files and other things.
+            """
             nonlocal next_file
 
+            # Miscellaneous rsync stdout lines to skip
             if not line:
                 return
+            if line.startswith(("building file list", "created directory", "sending")):
+                return
+            if line.startswith("sent "):
+                # sent 6,676 bytes  received 397 bytes  4,715.33 bytes/sec
+                return
+            if line.startswith("total "):
+                # total size is 315,265,653  speedup is 44,573.12 (DRY RUN)
+                return
+            if line.startswith(("cd", ".d")):
+                return
 
+            # Lines with chr(13), \r, contain file transfer information
             if chr(13) in line:
                 # partial transfer
                 #
@@ -287,15 +336,8 @@ class RSyncer(Observer):
                 successful_updates.append(next_file._replace(file_size=size_bytes))
                 next_file = None
                 return
-            if line.startswith(("building file list", "created directory", "sending")):
-                return
-            if line.startswith("sent "):
-                # sent 6,676 bytes  received 397 bytes  4,715.33 bytes/sec
-                return
-            if line.startswith("total "):
-                # total size is 315,265,653  speedup is 44,573.12 (DRY RUN)
-                return
 
+            # Lines starting with "*f" contain file transfer information
             if line.startswith((".f", ">f", "<f")):
                 # .d          ./
                 # .f          README.md
@@ -316,7 +358,9 @@ class RSyncer(Observer):
                     self._files_transferred - previously_transferred
                 )
                 update = RSyncerUpdate(
-                    file_path=Path(line[12:].replace(" ", "")),
+                    file_path=Path(
+                        line[12:].rstrip()
+                    ),  # Remove trailing newlines, spaces, and carriage returns
                     file_size=0,
                     outcome=TransferResult.SUCCESS,
                     transfer_total=self._files_transferred - previously_transferred,
@@ -333,18 +377,20 @@ class RSyncer(Observer):
                     next_file = update
                 return
 
-            if line.startswith(("cd", ".d")):
-                return
-
         def parse_stderr(line: str):
-            logger.warning(line)
+            if line.strip():
+                logger.warning(f"rsync stderr: {line!r}")
 
-        relative_filenames = []
+        # Generate list of relative filenames for this batch of transferred files
+        #   Relative filenames will be safe to use on both Windows and Unix
+        relative_filenames: List[Path] = []
         for f in files:
             try:
                 relative_filenames.append(f.relative_to(self._basepath))
             except ValueError:
                 raise ValueError(f"File '{f}' is outside of {self._basepath}") from None
+
+        # Encode files to rsync as bytestring
         if self._remove_files:
             if self._required_substrings_for_removal:
                 rsync_stdin_remove = b"\n".join(
@@ -371,49 +417,90 @@ class RSyncer(Observer):
         else:
             rsync_stdin_remove = b""
             rsync_stdin = b"\n".join(os.fsencode(f) for f in relative_filenames)
+
+        # Create and run rsync subprocesses
+        # rsync default settings
         rsync_cmd = [
             "rsync",
             "-iiv",
             "--times",
             "--progress",
             "--outbuf=line",
-            "--files-from=-",
-            "-p",  # preserve permissions
+            "--files-from=-",  # '-' indicates reading from standard input
+            # Needed as a pair to trigger permission modifications
+            # Ref: https://serverfault.com/a/796341
+            "-p",
+            "--chmod=D0750,F0750",  # Use extended chmod format
         ]
-
+        # Add file locations
         rsync_cmd.extend([".", self._remote])
 
+        # Transfer files to destination
+        result: subprocess.CompletedProcess[bytes] | None = None
         success = True
         if rsync_stdin:
-            result = procrunner.run(
-                rsync_cmd,
-                callback_stdout=parse_stdout,
-                callback_stderr=parse_stderr,
-                working_directory=str(self._basepath),
-                stdin=rsync_stdin,
-                print_stdout=False,
-                print_stderr=False,
+            # Wrap rsync command in a bash command
+            cmd = [
+                "bash",
+                "-c",
+                # rsync command passed in as a single string
+                " ".join(rsync_cmd),
+            ]
+            result = subprocess.run(
+                cmd,
+                cwd=self._basepath,  # As-is Path is fine
+                capture_output=True,
+                input=rsync_stdin,
             )
+            # Parse outputs
+            for line in result.stdout.decode("utf-8", "replace").split("\n"):
+                parse_stdout(line)
+            for line in result.stderr.decode("utf-8", "replace").split("\n"):
+                parse_stderr(line)
             success = result.returncode == 0
 
+        # Remove files from source
         if rsync_stdin_remove:
+            # Insert file removal flag before locations
             rsync_cmd.insert(-2, "--remove-source-files")
-            result = procrunner.run(
-                rsync_cmd,
-                callback_stdout=parse_stdout,
-                callback_stderr=parse_stderr,
-                working_directory=str(self._basepath),
-                stdin=rsync_stdin_remove,
-                print_stdout=False,
-                print_stderr=False,
+            # Wrap rsync command in a bash command
+            cmd = [
+                "bash",
+                "-c",
+                # Pass rsync command as single string
+                " ".join(rsync_cmd),
+            ]
+            result = subprocess.run(
+                cmd,
+                cwd=self._basepath,
+                capture_output=True,
+                input=rsync_stdin_remove,
             )
-
+            # Parse outputs
+            for line in result.stdout.decode("utf-8", "replace").split("\n"):
+                parse_stdout(line)
+            for line in result.stderr.decode("utf-8", "replace").split("\n"):
+                parse_stderr(line)
+            # Leave it as a failure if the previous rsync subprocess failed
             if success:
                 success = result.returncode == 0
 
         self.notify(successful_updates, secondary=True)
 
+        # Print out a summary message for each file transfer batch instead of individual messages
+        # List out file paths as stored in memory to see if issue is due to file path mismatch
+        if len(set(relative_filenames) - transfer_success) != 0:
+            logger.debug(
+                f"Files identified for transfer ({len(relative_filenames)}): {relative_filenames!r}"
+            )
+            logger.debug(
+                f"Files successfully transferred ({len(transfer_success)}): {list(transfer_success)!r}"
+            )
+
+        # Compare files from rsync stdout to original list to verify transfer
         for f in set(relative_filenames) - transfer_success:
+            # Mute individual file warnings; replace with summarised one above
+            # logger.warning(f"Transfer of file {f.name!r} considered a failure")
             self._files_transferred += 1
             current_outstanding = self.queue.unfinished_tasks - (
                 self._files_transferred - previously_transferred
@@ -429,8 +516,11 @@ class RSyncer(Observer):
             self.notify(update)
             success = False
 
-        logger.log(
-            logging.WARNING if result.returncode else logging.DEBUG,
-            f"rsync process finished with return code {result.returncode}",
-        )
+        if result is None:
+            logger.error(f"No rsync process ran for files: {files}")
+        else:
+            logger.log(
+                logging.WARNING if result.returncode else logging.DEBUG,
+                f"rsync process finished with return code {result.returncode}",
+            )
         return success
