@@ -1,6 +1,7 @@
 from __future__ import annotations
 
 import logging
+import subprocess
 from datetime import datetime
 from functools import partial
 from pathlib import Path
@@ -8,7 +9,6 @@ from queue import Queue
 from typing import Awaitable, Callable, Dict, List, OrderedDict, TypeVar
 from urllib.parse import urlparse
 
-import procrunner
 import requests
 from textual.app import App
 from textual.reactive import reactive
@@ -25,6 +25,7 @@ from murfey.client.tui.screens import (
     MainScreen,
     ProcessingForm,
     SessionSelection,
+    VisitCreation,
     VisitSelection,
     WaitingScreen,
     determine_default_destination,
@@ -32,12 +33,24 @@ from murfey.client.tui.screens import (
 from murfey.client.tui.status_bar import StatusBar
 from murfey.client.watchdir import DirWatcher
 from murfey.client.watchdir_multigrid import MultigridDirWatcher
-from murfey.util import capture_post, get_machine_config, set_default_acquisition_output
+from murfey.util import (
+    capture_post,
+    get_machine_config_client,
+    posix_path,
+    read_config,
+    set_default_acquisition_output,
+)
 
 log = logging.getLogger("murfey.tui.app")
 
 ReactiveType = TypeVar("ReactiveType")
 
+token = read_config()["Murfey"].get("token", "")
+
+requests.get = partial(requests.get, headers={"Authorization": f"Bearer {token}"})
+requests.post = partial(requests.post, headers={"Authorization": f"Bearer {token}"})
+requests.delete = partial(requests.delete, headers={"Authorization": f"Bearer {token}"})
+
 
 class MurfeyTUI(App):
     CSS_PATH = "controller.css"
@@ -61,7 +74,6 @@ class MurfeyTUI(App):
         gain_ref: Path | None = None,
         redirected_logger=None,
         force_mdoc_metadata: bool = False,
-        strict: bool = False,
         processing_enabled: bool = True,
         skip_existing_processing: bool = False,
         **kwargs,
@@ -91,10 +103,11 @@ class MurfeyTUI(App):
         self._processing_enabled = processing_enabled
         self._multigrid_watcher: MultigridDirWatcher | None = None
         self._force_mdoc_metadata = force_mdoc_metadata
-        self._strict = strict
         self._skip_existing_processing = skip_existing_processing
-        self._machine_config = get_machine_config(
-            str(self._environment.url.geturl()), demo=self._environment.demo
+        self._machine_config = get_machine_config_client(
+            str(self._environment.url.geturl()),
+            instrument_name=self._environment.instrument_name,
+            demo=self._environment.demo,
         )
         self._data_suffixes = (".mrc", ".tiff", ".tif", ".eer")
         self._data_substrings = [
@@ -105,18 +118,14 @@ class MurfeyTUI(App):
         ]
         self.install_screen(MainScreen(), "main")
 
-    @property
-    def role(self) -> str:
-        if self.analyser:
-            return self.analyser._role
-        return ""
-
     def _launch_multigrid_watcher(
         self, source: Path, destination_overrides: Dict[Path, str] | None = None
     ):
         log.info(f"Launching multigrid watcher for source {source}")
-        machine_config = get_machine_config(
-            str(self._environment.url.geturl()), demo=self._environment.demo
+        machine_config = get_machine_config_client(
+            str(self._environment.url.geturl()),
+            instrument_name=self._environment.instrument_name,
+            demo=self._environment.demo,
         )
         self._multigrid_watcher = MultigridDirWatcher(
             source,
@@ -141,10 +150,11 @@ class MurfeyTUI(App):
         remove_files: bool = False,
         analyse: bool = True,
         limited: bool = False,
+        **kwargs,
     ):
         log.info(f"starting multigrid rsyncer: {source}")
         destination_overrides = destination_overrides or {}
-        machine_data = requests.get(f"{self._environment.url.geturl()}/machine/").json()
+        machine_data = requests.get(f"{self._environment.url.geturl()}/machine").json()
         if destination_overrides.get(source):
             destination = destination_overrides[source] + f"/{extra_directory}"
         else:
@@ -153,9 +163,9 @@ class MurfeyTUI(App):
                     destination = str(k / extra_directory)
                     break
             else:
-                self._environment.default_destinations[
-                    source
-                ] = f"{machine_data.get('rsync_module') or 'data'}/{datetime.now().year}"
+                self._environment.default_destinations[source] = (
+                    f"{machine_data.get('rsync_module') or 'data'}/{datetime.now().year}"
+                )
                 destination = determine_default_destination(
                     self._visit,
                     source,
@@ -176,6 +186,8 @@ class MurfeyTUI(App):
             analyse=analyse,
             remove_files=remove_files,
             limited=limited,
+            transfer=machine_data.get("data_transfer_enabled", True),
+            rsync_url=machine_data.get("rsync_url", ""),
         )
 
     def _start_rsyncer(
@@ -187,71 +199,86 @@ class MurfeyTUI(App):
         analyse: bool = True,
         remove_files: bool = False,
         limited: bool = False,
+        transfer: bool = True,
+        rsync_url: str = "",
     ):
         log.info(f"starting rsyncer: {source}")
+        if transfer:
+            # Always make sure the destination directory exists
+            make_directory_url = f"{str(self._url.geturl())}/sessions/{str(self._environment.murfey_session)}/make_rsyncer_destination"
+            capture_post(make_directory_url, json={"destination": destination})
         if self._environment:
             self._environment.default_destinations[source] = destination
             if self._environment.gain_ref and visit_path:
-                gain_rsync = procrunner.run(
-                    [
-                        "rsync",
-                        str(self._environment.gain_ref),
-                        f"{self._url.hostname}::{visit_path}/processing",
-                    ]
-                )
+                # Set up rsync command
+                rsync_cmd = [
+                    "rsync",
+                    f"{posix_path(self._environment.gain_ref)!r}",
+                    f"{self._url.hostname}::{visit_path}/processing",
+                ]
+                # Encase in bash shell
+                cmd = [
+                    "bash",
+                    "-c",
+                    " ".join(rsync_cmd),
+                ]
+                # Run rsync subprocess
+                gain_rsync = subprocess.run(cmd)
                 if gain_rsync.returncode:
                     log.warning(
-                        f"Gain reference file {self._environment.gain_ref} was not successfully transferred to {visit_path}/processing"
+                        f"Gain reference file {posix_path(self._environment.gain_ref)!r} was not successfully transferred to {visit_path}/processing"
                     )
-        self.rsync_processes[source] = RSyncer(
-            source,
-            basepath_remote=Path(destination),
-            server_url=self._url,
-            # local=self._environment.demo,
-            status_bar=self._statusbar,
-            do_transfer=self._do_transfer,
-            required_substrings_for_removal=self._data_substrings,
-            remove_files=remove_files,
-        )
-
-        def rsync_result(update: RSyncerUpdate):
-            if not update.base_path:
-                raise ValueError("No base path from rsyncer update")
-            if not self.rsync_processes.get(update.base_path):
-                raise ValueError("TUI rsync process does not exist")
-            if update.outcome is TransferResult.SUCCESS:
-                # log.info(
-                #     f"File {str(update.file_path)!r} successfully transferred ({update.file_size} bytes)"
-                # )
-                pass
-            else:
-                log.warning(f"Failed to transfer file {str(update.file_path)!r}")
-                self.rsync_processes[update.base_path].enqueue(update.file_path)
-
-        self.rsync_processes[source].subscribe(rsync_result)
-        self.rsync_processes[source].subscribe(
-            partial(
-                self._increment_transferred_files_prometheus,
-                destination=destination,
-                source=str(source),
+        if transfer:
+            self.rsync_processes[source] = RSyncer(
+                source,
+                basepath_remote=Path(destination),
+                server_url=urlparse(rsync_url) if rsync_url else self._url,
+                # local=self._environment.demo,
+                status_bar=self._statusbar,
+                do_transfer=self._do_transfer,
+                required_substrings_for_removal=self._data_substrings,
+                remove_files=remove_files,
             )
-        )
-        self.rsync_processes[source].subscribe(
-            partial(
-                self._increment_transferred_files,
-                destination=destination,
-                source=str(source),
-            ),
-            secondary=True,
-        )
-        url = f"{str(self._url.geturl())}/visits/{str(self._visit)}/rsyncer"
-        rsyncer_data = {
-            "source": str(source),
-            "destination": destination,
-            "client_id": self._environment.client_id,
-            "transferring": self._do_transfer,
-        }
-        requests.post(url, json=rsyncer_data)
+
+            def rsync_result(update: RSyncerUpdate):
+                if not update.base_path:
+                    raise ValueError("No base path from rsyncer update")
+                if not self.rsync_processes.get(update.base_path):
+                    raise ValueError("TUI rsync process does not exist")
+                if update.outcome is TransferResult.SUCCESS:
+                    log.debug(
+                        f"Succesfully transferred file {str(update.file_path)!r} ({update.file_size} bytes)"
+                    )
+                    # pass
+                else:
+                    log.warning(f"Failed to transfer file {str(update.file_path)!r}")
+                    self.rsync_processes[update.base_path].enqueue(update.file_path)
+
+            self.rsync_processes[source].subscribe(rsync_result)
+            self.rsync_processes[source].subscribe(
+                partial(
+                    self._increment_transferred_files_prometheus,
+                    destination=destination,
+                    source=str(source),
+                )
+            )
+            self.rsync_processes[source].subscribe(
+                partial(
+                    self._increment_transferred_files,
+                    destination=destination,
+                    source=str(source),
+                ),
+                secondary=True,
+            )
+            url = f"{str(self._url.geturl())}/sessions/{str(self._environment.murfey_session)}/rsyncer"
+            rsyncer_data = {
+                "source": str(source),
+                "destination": destination,
+                "session_id": self._environment.murfey_session,
+                "transferring": self._do_transfer,
+            }
+            requests.post(url, json=rsyncer_data)
+
         self._environment.watchers[source] = DirWatcher(source, settling_time=30)
 
         if not self.analysers.get(source) and analyse:
@@ -262,16 +289,6 @@ class MurfeyTUI(App):
                 force_mdoc_metadata=self._force_mdoc_metadata,
                 limited=limited,
             )
-            machine_data = requests.get(
-                f"{self._environment.url.geturl()}/machine/"
-            ).json()
-            for data_dir in machine_data["data_directories"].keys():
-                if source.resolve().is_relative_to(Path(data_dir)):
-                    self.analysers[source]._role = machine_data["data_directories"][
-                        data_dir
-                    ]
-                    log.info(f"role found for {source}")
-                    break
             if force_metadata:
                 self.analysers[source].subscribe(
                     partial(self._start_dc, from_form=True)
@@ -279,15 +296,36 @@ class MurfeyTUI(App):
             else:
                 self.analysers[source].subscribe(self._data_collection_form)
             self.analysers[source].start()
-            self.rsync_processes[source].subscribe(self.analysers[source].enqueue)
+            if transfer:
+                self.rsync_processes[source].subscribe(self.analysers[source].enqueue)
 
-        self.rsync_processes[source].start()
+        if transfer:
+            self.rsync_processes[source].start()
 
         if self._environment:
             if self._environment.watchers.get(source):
-                self._environment.watchers[source].subscribe(
-                    self.rsync_processes[source].enqueue
-                )
+                if transfer:
+                    self._environment.watchers[source].subscribe(
+                        self.rsync_processes[source].enqueue
+                    )
+                else:
+
+                    def _rsync_update_converter(p: Path) -> None:
+                        self.analysers[source].enqueue(
+                            RSyncerUpdate(
+                                file_path=p,
+                                file_size=0,
+                                outcome=TransferResult.SUCCESS,
+                                transfer_total=0,
+                                queue_size=0,
+                                base_path=source,
+                            )
+                        )
+                        return None
+
+                    self._environment.watchers[source].subscribe(
+                        _rsync_update_converter
+                    )
                 self._environment.watchers[source].subscribe(
                     partial(
                         self._increment_file_count,
@@ -314,7 +352,7 @@ class MurfeyTUI(App):
             data = {
                 "source": source,
                 "destination": destination,
-                "client_id": self._environment.client_id,
+                "session_id": self._environment.murfey_session,
                 "increment_count": len(observed_files),
                 "increment_data_count": num_data_files,
             }
@@ -339,7 +377,7 @@ class MurfeyTUI(App):
             data = {
                 "source": source,
                 "destination": destination,
-                "client_id": self._environment.client_id,
+                "session_id": self._environment.murfey_session,
                 "increment_count": 1,
                 "bytes": update.file_size,
                 "increment_data_count": len(data_files),
@@ -367,7 +405,7 @@ class MurfeyTUI(App):
         data = {
             "source": source,
             "destination": destination,
-            "client_id": self._environment.client_id,
+            "session_id": self._environment.murfey_session,
             "increment_count": len(checked_updates),
             "bytes": sum(f.file_size for f in checked_updates),
             "increment_data_count": len(data_files),
@@ -402,9 +440,9 @@ class MurfeyTUI(App):
                 f"gain reference is set to {self._form_values.get('gain_ref')}, {self._environment.data_collection_parameters.get('gain_ref')}"
             )
             if self._form_values.get("gain_ref") in (None, "None"):
-                self._form_values[
-                    "gain_ref"
-                ] = self._environment.data_collection_parameters.get("gain_ref")
+                self._form_values["gain_ref"] = (
+                    self._environment.data_collection_parameters.get("gain_ref")
+                )
             self._form_dependencies = response.get("dependencies", {})
             self.processing_btn.disabled = False
             self._data_collection_form_complete = True
@@ -436,17 +474,21 @@ class MurfeyTUI(App):
         context = self.analysers[source]._context
         if isinstance(context, TomographyContext):
             source = Path(json["source"])
-            url = f"{str(self._url.geturl())}/visits/{str(self._visit)}/{self._environment.client_id}/register_data_collection_group"
+            url = f"{str(self._url.geturl())}/visits/{str(self._visit)}/{self._environment.murfey_session}/register_data_collection_group"
             dcg_data = {
                 "experiment_type": "tomo",
                 "experiment_type_id": 36,
                 "tag": str(source),
-                "atlas": str(self._environment.samples[source].atlas)
-                if self._environment.samples.get(source)
-                else "",
-                "sample": self._environment.samples[source].sample
-                if self._environment.samples.get(source)
-                else None,
+                "atlas": (
+                    str(self._environment.samples[source].atlas)
+                    if self._environment.samples.get(source)
+                    else ""
+                ),
+                "sample": (
+                    self._environment.samples[source].sample
+                    if self._environment.samples.get(source)
+                    else None
+                ),
             }
             capture_post(url, json=dcg_data)
             data = {
@@ -468,38 +510,64 @@ class MurfeyTUI(App):
                 "phase_plate": json.get("phase_plate", False),
             }
             capture_post(
-                f"{str(self._url.geturl())}/visits/{str(self._visit)}/{self._environment.client_id}/start_data_collection",
+                f"{str(self._url.geturl())}/visits/{str(self._visit)}/{self._environment.murfey_session}/start_data_collection",
                 json=data,
             )
             for recipe in ("em-tomo-preprocess", "em-tomo-align"):
                 capture_post(
-                    f"{str(self._url.geturl())}/visits/{str(self._visit)}/{self._environment.client_id}/register_processing_job",
-                    json={"tag": json["tilt_series_tag"], "recipe": recipe},
+                    f"{str(self._url.geturl())}/visits/{str(self._visit)}/{self._environment.murfey_session}/register_processing_job",
+                    json={
+                        "tag": json["tilt_series_tag"],
+                        "source": str(source),
+                        "recipe": recipe,
+                    },
                 )
             log.info("Registering tomography processing parameters")
+            if self.app._environment.data_collection_parameters.get("num_eer_frames"):
+                eer_response = requests.post(
+                    f"{str(self.app._environment.url.geturl())}/visits/{self.app._environment.visit}/{self.app._environment.murfey_session}/eer_fractionation_file",
+                    json={
+                        "num_frames": self.app._environment.data_collection_parameters[
+                            "num_eer_frames"
+                        ],
+                        "fractionation": self.app._environment.data_collection_parameters[
+                            "eer_fractionation"
+                        ],
+                        "dose_per_frame": self.app._environment.data_collection_parameters[
+                            "dose_per_frame"
+                        ],
+                        "fractionation_file_name": "eer_fractionation_tomo.txt",
+                    },
+                )
+                eer_fractionation_file = eer_response.json()["eer_fractionation_file"]
+                json.update({"eer_fractionation_file": eer_fractionation_file})
             requests.post(
-                f"{self.app._environment.url.geturl()}/clients/{self.app._environment.client_id}/tomography_preprocessing_parameters",
+                f"{self.app._environment.url.geturl()}/sessions/{self.app._environment.murfey_session}/tomography_preprocessing_parameters",
                 json=json,
             )
             context._flush_data_collections()
             context._flush_processing_jobs()
             capture_post(
-                f"{self.app._environment.url.geturl()}/visits/{self._visit}/{self.app._environment.client_id}/flush_tomography_processing",
+                f"{self.app._environment.url.geturl()}/visits/{self._visit}/{self.app._environment.murfey_session}/flush_tomography_processing",
                 json={"rsync_source": str(source)},
             )
             log.info("tomography processing flushed")
         elif isinstance(context, SPAContext) or isinstance(context, SPAModularContext):
-            url = f"{str(self._url.geturl())}/visits/{str(self._visit)}/{self._environment.client_id}/register_data_collection_group"
+            url = f"{str(self._url.geturl())}/visits/{str(self._visit)}/{self._environment.murfey_session}/register_data_collection_group"
             dcg_data = {
                 "experiment_type": "single particle",
                 "experiment_type_id": 37,
                 "tag": str(source),
-                "atlas": str(self._environment.samples[source].atlas)
-                if self._environment.samples.get(source)
-                else "",
-                "sample": self._environment.samples[source].sample
-                if self._environment.samples.get(source)
-                else None,
+                "atlas": (
+                    str(self._environment.samples[source].atlas)
+                    if self._environment.samples.get(source)
+                    else ""
+                ),
+                "sample": (
+                    self._environment.samples[source].sample
+                    if self._environment.samples.get(source)
+                    else None
+                ),
             }
             capture_post(url, json=dcg_data)
             if from_form:
@@ -524,7 +592,7 @@ class MurfeyTUI(App):
                     "phase_plate": json.get("phase_plate", False),
                 }
                 capture_post(
-                    f"{str(self._url.geturl())}/visits/{str(self._visit)}/{self._environment.client_id}/start_data_collection",
+                    f"{str(self._url.geturl())}/visits/{str(self._visit)}/{self._environment.murfey_session}/start_data_collection",
                     json=data,
                 )
                 for recipe in (
@@ -535,12 +603,16 @@ class MurfeyTUI(App):
                     "em-spa-refine",
                 ):
                     capture_post(
-                        f"{str(self._url.geturl())}/visits/{str(self._visit)}/{self._environment.client_id}/register_processing_job",
-                        json={"tag": str(source), "recipe": recipe},
+                        f"{str(self._url.geturl())}/visits/{str(self._visit)}/{self._environment.murfey_session}/register_processing_job",
+                        json={
+                            "tag": str(source),
+                            "source": str(source),
+                            "recipe": recipe,
+                        },
                     )
                 log.info(f"Posting SPA processing parameters: {json}")
                 response = capture_post(
-                    f"{self.app._environment.url.geturl()}/clients/{self.app._environment.client_id}/spa_processing_parameters",
+                    f"{self.app._environment.url.geturl()}/sessions/{self.app._environment.murfey_session}/spa_processing_parameters",
                     json={
                         **{k: None if v == "None" else v for k, v in json.items()},
                         "tag": str(source),
@@ -554,11 +626,11 @@ class MurfeyTUI(App):
                 if not str(response.status_code).startswith("2"):
                     log.warning(f"{response.reason}")
                 capture_post(
-                    f"{self.app._environment.url.geturl()}/visits/{self.app._environment.visit}/{self.app._environment.client_id}/flush_spa_processing",
+                    f"{self.app._environment.url.geturl()}/visits/{self.app._environment.visit}/{self.app._environment.murfey_session}/flush_spa_processing",
                     json={"tag": str(source)},
                 )
             if isinstance(context, SPAContext):
-                url = f"{str(self._url.geturl())}/visits/{str(self._visit)}/{self._environment.client_id}/start_data_collection"
+                url = f"{str(self._url.geturl())}/visits/{str(self._visit)}/{self._environment.murfey_session}/start_data_collection"
                 self._environment.listeners["data_collection_group_ids"] = {
                     partial(
                         context._register_data_collection,
@@ -615,8 +687,12 @@ class MurfeyTUI(App):
         exisiting_sessions = requests.get(
             f"{self._environment.url.geturl()}/sessions"
         ).json()
-        self.install_screen(VisitSelection(self.visits), "visit-select-screen")
-        self.push_screen("visit-select-screen")
+        if self.visits:
+            self.install_screen(VisitSelection(self.visits), "visit-select-screen")
+            self.push_screen("visit-select-screen")
+        else:
+            self.install_screen(VisitCreation(), "visit-creation-screen")
+            self.push_screen("visit-creation-screen")
         if exisiting_sessions:
             self.install_screen(
                 SessionSelection(
@@ -636,7 +712,7 @@ class MurfeyTUI(App):
         else:
             session_name = "Client connection"
             resp = capture_post(
-                f"{self._environment.url.geturl()}/clients/{self._environment.client_id}/session",
+                f"{self._environment.url.geturl()}/instruments/{self._environment.instrument_name}/clients/{self._environment.client_id}/session",
                 json={"session_id": None, "session_name": session_name},
             )
             if resp:
@@ -646,14 +722,16 @@ class MurfeyTUI(App):
         self.log_book.write(message.renderable)
 
     async def reset(self):
-        machine_config = get_machine_config(
-            str(self._environment.url.geturl()), demo=self._environment.demo
+        machine_config = get_machine_config_client(
+            str(self._environment.url.geturl()),
+            instrument_name=self._environment.instrument_name,
+            demo=self._environment.demo,
         )
         if self.rsync_processes and machine_config.get("allow_removal"):
             sources = "\n".join(str(k) for k in self.rsync_processes.keys())
             prompt = f"Remove files from the following:\n {sources} \n"
             rsync_instances = requests.get(
-                f"{self._environment.url.geturl()}/clients/{self._environment.client_id}/rsyncers"
+                f"{self._environment.url.geturl()}/sessions/{self._environment.murfey_session}/rsyncers"
             ).json()
             prompt += f"Copied {sum(r['files_counted'] for r in rsync_instances)} / {sum(r['files_transferred'] for r in rsync_instances)}"
             self.install_screen(
@@ -674,11 +752,10 @@ class MurfeyTUI(App):
         if self._multigrid_watcher:
             self._multigrid_watcher.stop()
         self.exit()
-        exit()
 
     async def action_remove_session(self) -> None:
         requests.delete(
-            f"{self._environment.url.geturl()}/clients/{self._environment.client_id}/session"
+            f"{self._environment.url.geturl()}/instruments/{self._environment.instrument_name}/clients/{self._environment.client_id}/session"
         )
         if self.rsync_processes:
             for rp in self.rsync_processes.values():
@@ -689,18 +766,18 @@ class MurfeyTUI(App):
         if self._multigrid_watcher:
             self._multigrid_watcher.stop()
         self.exit()
-        exit()
 
     def clean_up_quit(self) -> None:
         requests.delete(
-            f"{self._environment.url.geturl()}/clients/{self._environment.client_id}/session"
+            f"{self._environment.url.geturl()}/instruments/{self._environment.instrument_name}/clients/{self._environment.client_id}/session"
         )
         self.exit()
-        exit()
 
     async def action_clear(self) -> None:
-        machine_config = get_machine_config(
-            str(self._environment.url.geturl()), demo=self._environment.demo
+        machine_config = get_machine_config_client(
+            str(self._environment.url.geturl()),
+            instrument_name=self._environment.instrument_name,
+            demo=self._environment.demo,
         )
         if self.rsync_processes and machine_config.get("allow_removal"):
             sources = "\n".join(str(k) for k in self.rsync_processes.keys())
@@ -735,13 +812,12 @@ class MurfeyTUI(App):
             removal_rp.stop()
             log.info(f"rsyncer {rp} rerun with removal")
         requests.post(
-            f"{self._environment.url.geturl()}/clients/{self._environment.client_id}/successful_processing"
+            f"{self._environment.url.geturl()}/sessions/{self._environment.murfey_session}/successful_processing"
         )
         requests.delete(
-            f"{self._environment.url.geturl()}/clients/{self._environment.client_id}/session"
+            f"{self._environment.url.geturl()}/instruments/{self._environment.instrument_name}/clients/{self._environment.client_id}/session"
         )
         self.exit()
-        exit()
 
     async def action_process(self) -> None:
         self.processing_btn.disabled = False
