@@ -2,6 +2,7 @@ from __future__ import annotations
 
 # import contextlib
 import logging
+import subprocess
 from datetime import datetime
 from functools import partial
 from pathlib import Path
@@ -17,7 +18,6 @@ from typing import (
     TypeVar,
 )
 
-import procrunner
 import requests
 from pydantic import BaseModel, ValidationError
 from rich.box import SQUARE
@@ -56,13 +56,19 @@ from murfey.client.instance_environment import (
 )
 from murfey.client.rsync import RSyncer
 from murfey.client.tui.forms import FormDependency
-from murfey.util import capture_post, get_machine_config
+from murfey.util import capture_post, get_machine_config_client, posix_path, read_config
 from murfey.util.models import PreprocessingParametersTomo, ProcessingParametersSPA
 
 log = logging.getLogger("murfey.tui.screens")
 
 ReactiveType = TypeVar("ReactiveType")
 
+token = read_config()["Murfey"].get("token", "")
+
+requests.get = partial(requests.get, headers={"Authorization": f"Bearer {token}"})
+requests.post = partial(requests.post, headers={"Authorization": f"Bearer {token}"})
+requests.delete = partial(requests.delete, headers={"Authorization": f"Bearer {token}"})
+
 
 def determine_default_destination(
     visit: str,
@@ -75,25 +81,22 @@ def determine_default_destination(
     include_mid_path: bool = True,
     use_suggested_path: bool = True,
 ) -> str:
-    machine_data = requests.get(f"{environment.url.geturl()}/machine/").json()
+    machine_data = requests.get(
+        f"{environment.url.geturl()}/instruments/{environment.instrument_name}/machine"
+    ).json()
     _default = ""
     if environment.processing_only_mode and environment.sources:
         log.info(f"Processing only mode with sources {environment.sources}")
         _default = str(environment.sources[0].resolve()) or str(Path.cwd())
     elif machine_data.get("data_directories"):
-        for data_dir in machine_data["data_directories"].keys():
+        for data_dir in machine_data["data_directories"]:
             if source.resolve() == Path(data_dir):
                 _default = destination + f"/{visit}"
-                if analysers.get(source):
-                    analysers[source]._role = machine_data["data_directories"][data_dir]
                 break
             else:
                 try:
                     mid_path = source.resolve().relative_to(data_dir)
-                    if (
-                        machine_data["data_directories"][data_dir] == "detector"
-                        and use_suggested_path
-                    ):
+                    if use_suggested_path:
                         with global_env_lock:
                             source_name = (
                                 source.name
@@ -104,7 +107,7 @@ def determine_default_destination(
                                 _default = environment.destination_registry[source_name]
                             else:
                                 suggested_path_response = capture_post(
-                                    url=f"{str(environment.url.geturl())}/visits/{visit}/suggested_path",
+                                    url=f"{str(environment.url.geturl())}/visits/{visit}/{environment.murfey_session}/suggested_path",
                                     json={
                                         "base_path": f"{destination}/{visit}/{mid_path.parent if include_mid_path else ''}/raw",
                                         "touch": touch,
@@ -119,10 +122,6 @@ def determine_default_destination(
                                 environment.destination_registry[source_name] = _default
                     else:
                         _default = f"{destination}/{visit}/{mid_path if include_mid_path else source.name}"
-                    if analysers.get(source):
-                        analysers[source]._role = machine_data["data_directories"][
-                            data_dir
-                        ]
                     break
                 except (ValueError, KeyError):
                     _default = ""
@@ -130,7 +129,11 @@ def determine_default_destination(
             _default = ""
     else:
         _default = destination + f"/{visit}"
-    return _default + f"/{extra_directory}"
+    return (
+        _default + f"/{extra_directory}"
+        if not _default.endswith("/")
+        else _default + f"{extra_directory}"
+    )
 
 
 class InputResponse(NamedTuple):
@@ -201,10 +204,10 @@ def validate_form(form: dict, model: BaseModel) -> bool:
 class _DirectoryTree(DirectoryTree):
     valid_selection = reactive(False)
 
-    def __init__(self, *args, data_directories: dict | None = None, **kwargs):
+    def __init__(self, *args, data_directories: List[Path] | None = None, **kwargs):
         super().__init__(*args, **kwargs)
         self._selected_path = self.path
-        self._data_directories = data_directories or {}
+        self._data_directories = data_directories or []
 
     def on_tree_node_selected(self, event: Tree.NodeSelected) -> None:
         event.stop()
@@ -254,12 +257,14 @@ class LaunchScreen(Screen):
         super().__init__(*args, **kwargs)
         self._selected_dir = basepath
         self._add_basepath = add_basepath
-        cfg = get_machine_config(
-            str(self.app._environment.url.geturl()), demo=self.app._environment.demo
+        cfg = get_machine_config_client(
+            str(self.app._environment.url.geturl()),
+            instrument_name=self.app._environment.instrument_name,
+            demo=self.app._environment.demo,
+        )
+        self._context: (
+            Type[SPAModularContext] | Type[SPAContext] | Type[TomographyContext]
         )
-        self._context: Type[SPAModularContext] | Type[SPAContext] | Type[
-            TomographyContext
-        ]
         if cfg.get("modular_spa"):
             self._context = SPAContext
         else:
@@ -267,13 +272,11 @@ class LaunchScreen(Screen):
 
     def compose(self):
         machine_data = requests.get(
-            f"{self.app._environment.url.geturl()}/machine/"
+            f"{self.app._environment.url.geturl()}/machine"
         ).json()
         self._dir_tree = _DirectoryTree(
             str(self._selected_dir),
-            data_directories=machine_data.get("data_directories", {})
-            if self.app._strict
-            else {},
+            data_directories=machine_data.get("data_directories", []),
             id="dir-select",
         )
 
@@ -285,7 +288,7 @@ class LaunchScreen(Screen):
 
         text_log.write("Selected directories:\n")
         btn_disabled = True
-        for d in machine_data.get("data_directories", {}).keys():
+        for d in machine_data.get("data_directories", []):
             if (
                 Path(self._dir_tree._selected_path).resolve().is_relative_to(d)
                 or self.app._environment.processing_only_mode
@@ -318,11 +321,11 @@ class LaunchScreen(Screen):
                     return
             self.app._environment.sources.append(source)
             machine_data = requests.get(
-                f"{self.app._environment.url.geturl()}/machine/"
+                f"{self.app._environment.url.geturl()}/machine"
             ).json()
-            self.app._default_destinations[
-                source
-            ] = f"{machine_data.get('rsync_module') or 'data'}/{datetime.now().year}"
+            self.app._default_destinations[source] = (
+                f"{machine_data.get('rsync_module') or 'data'}/{datetime.now().year}"
+            )
         if self._launch_btn:
             self._launch_btn.disabled = False
         self.query_one("#selected-directories").write(str(source) + "\n")
@@ -343,6 +346,7 @@ class LaunchScreen(Screen):
                     defd,
                     self.app._environment,
                     self.app.analysers,
+                    touch=True,
                 )
                 visit_path = defd + f"/{text}"
                 if self.app._environment.processing_only_mode:
@@ -458,7 +462,7 @@ class ProcessingForm(Screen):
         if self._form.get("motion_corr_binning") == "2":
             self._vert = VerticalScroll(
                 *inputs,
-                Label("Collected in super resoultion mode unbinned:"),
+                Label("Collected in counting mode:"),
                 Switch(id="superres", value=True, classes="input"),
                 confirm_btn,
                 id="input-form",
@@ -482,16 +486,16 @@ class ProcessingForm(Screen):
             self.app._start_dc(params)
             if model == PreprocessingParametersTomo:
                 requests.post(
-                    f"{self.app._environment.url.geturl()}/clients/{self.app._environment.client_id}/tomography_preprocessing_parameters",
+                    f"{self.app._environment.url.geturl()}/sessions/{self.app._environment.murfey_session}/tomography_preprocessing_parameters",
                     json=params,
                 )
             elif model == ProcessingParametersSPA:
                 requests.post(
-                    f"{self.app._environment.url.geturl()}/clients/{self.app._environment.client_id}/spa_processing_parameters",
+                    f"{self.app._environment.url.geturl()}/sessions/{self.app._environment.murfey_session}/spa_processing_parameters",
                     json=params,
                 )
                 requests.post(
-                    f"{self.app._environment.url.geturl()}/visits/{self.app._environment.visit}/{self.app._environment.client_id}/flush_spa_processing"
+                    f"{self.app._environment.url.geturl()}/visits/{self.app._environment.visit}/{self.app._environment.murfey_session}/flush_spa_processing"
                 )
 
     def on_switch_changed(self, event):
@@ -630,9 +634,11 @@ class SessionSelection(Screen):
             self.app.pop_screen()
             self.app.install_screen(
                 ConfirmScreen(
-                    f"Remove session {session_id} [WARNING: there are clients already using this session]"
-                    if str(event.button.label) in self._sessions_with_client
-                    else f"Remove session {session_id}",
+                    (
+                        f"Remove session {session_id} [WARNING: there are clients already using this session]"
+                        if str(event.button.label) in self._sessions_with_client
+                        else f"Remove session {session_id}"
+                    ),
                     pressed_callback=partial(self._remove_session, session_id),
                     button_names={"launch": "Yes"},
                     push="visit-select-screen",
@@ -649,7 +655,7 @@ class SessionSelection(Screen):
             self.app.pop_screen()
         session_name = "Client connection"
         self.app._environment.murfey_session = requests.post(
-            f"{self.app._environment.url.geturl()}/clients/{self.app._environment.client_id}/session",
+            f"{self.app._environment.url.geturl()}/instruments/{self.app._environment.instrument_name}/clients/{self.app._environment.client_id}/session",
             json={"session_id": session_id, "session_name": session_name},
         ).json()
 
@@ -678,7 +684,7 @@ class SessionSelection(Screen):
         else:
             session_name = "Client connection"
             resp = capture_post(
-                f"{self.app._environment.url.geturl()}/clients/{self.app._environment.client_id}/session",
+                f"{self.app._environment.url.geturl()}/instruments/{self._environment.instrument_name}/clients/{self.app._environment.client_id}/session",
                 json={"session_id": None, "session_name": session_name},
             )
             if resp:
@@ -705,15 +711,16 @@ class VisitSelection(SwitchSelection):
         )
         log.info(f"Posted visit registration: {response.status_code}")
         machine_data = requests.get(
-            f"{self.app._environment.url.geturl()}/machine/"
+            f"{self.app._environment.url.geturl()}/machine"
         ).json()
+
         if self._switch_status:
             self.app.install_screen(
                 DirectorySelection(
                     [
-                        p[0]
-                        for p in machine_data.get("data_directories", {}).items()
-                        if p[1] == "detector" and Path(p[0]).exists()
+                        p
+                        for p in machine_data.get("data_directories", [])
+                        if Path(p).exists()
                     ]
                 ),
                 "directory-select",
@@ -736,6 +743,121 @@ class VisitSelection(SwitchSelection):
                 self.app.install_screen(LaunchScreen(basepath=Path("./")), "launcher")
                 self.app.push_screen("launcher")
 
+        if machine_data.get("upstream_data_directories"):
+            upstream_downloads = requests.get(
+                f"{self.app._environment.url.geturl()}/sessions/{self.app._environment.murfey_session}/upstream_visits"
+            ).json()
+            self.app.install_screen(
+                UpstreamDownloads(upstream_downloads), "upstream-downloads"
+            )
+            self.app.push_screen("upstream-downloads")
+
+
+class VisitCreation(Screen):
+    # This allows for the manual creation of a visit name when there is no LIMS system to provide it
+    # Shares a lot of code with VisitSelection, should be neatened up at some point
+    visit_name: reactive[str] = reactive("")
+
+    def __init__(self, *args, **kwargs):
+        super().__init__(*args, **kwargs)
+
+    def compose(self):
+        yield Input(placeholder="Visit name", classes="input-visit-name")
+        yield Button("Create visit", classes="btn-visit-create")
+
+    def on_input_changed(self, event):
+        self.visit_name = event.value
+
+    def on_button_pressed(self, event: Button.Pressed):
+        text = str(self.visit_name)
+        self.app._visit = text
+        self.app._environment.visit = text
+        response = requests.post(
+            f"{self.app._environment.url.geturl()}/visits/{text}",
+            json={"id": self.app._environment.client_id},
+        )
+        log.info(f"Posted visit registration: {response.status_code}")
+        machine_data = requests.get(
+            f"{self.app._environment.url.geturl()}/machine"
+        ).json()
+
+        self.app.install_screen(
+            DirectorySelection(
+                [p for p in machine_data.get("data_directories", []) if p.exists()]
+            ),
+            "directory-select",
+        )
+        self.app.pop_screen()
+
+        if machine_data.get("gain_reference_directory"):
+            self.app.install_screen(
+                GainReference(
+                    determine_gain_ref(Path(machine_data["gain_reference_directory"])),
+                    True,
+                ),
+                "gain-ref-select",
+            )
+            self.app.push_screen("gain-ref-select")
+        else:
+            if self._switch_status:
+                self.app.push_screen("directory-select")
+            else:
+                self.app.install_screen(LaunchScreen(basepath=Path("./")), "launcher")
+                self.app.push_screen("launcher")
+
+        if machine_data.get("upstream_data_directories"):
+            upstream_downloads = requests.get(
+                f"{self.app._environment.url.geturl()}/sessions/{self.app._environment.murfey_session}/upstream_visits"
+            ).json()
+            self.app.install_screen(
+                UpstreamDownloads(upstream_downloads), "upstream-downloads"
+            )
+            self.app.push_screen("upstream-downloads")
+
+
+class UpstreamDownloads(Screen):
+    def __init__(self, connected_visits: Dict[str, Path], *args, **kwargs):
+        super().__init__(*args, **kwargs)
+        self._connected_visits = connected_visits
+
+    def compose(self):
+        visit_buttons = [
+            Button(cv, classes="btn-directory") for cv in self._connected_visits.keys()
+        ]
+        yield VerticalScroll(*visit_buttons)
+        yield Button("Skip", classes="btn-directory")
+
+    def on_button_pressed(self, event: Button.Pressed):
+        machine_data = requests.get(
+            f"{self.app._environment.url.geturl()}/machine"
+        ).json()
+        if machine_data.get("upstream_data_download_directory"):
+            # Create the directory locally to save files to
+            download_dir = Path(machine_data["upstream_data_download_directory"]) / str(
+                event.button.label
+            )
+            download_dir.mkdir(exist_ok=True)
+
+            # Get the paths to the TIFF files generated previously under the same session ID
+            upstream_tiff_paths_response = requests.get(
+                f"{self.app._environment.url.geturl()}/visits/{event.button.label}/{self.app._environment.murfey_session}/upstream_tiff_paths"
+            )
+            upstream_tiff_paths = upstream_tiff_paths_response.json() or []
+
+            # Request to download the TIFF files found
+            for tp in upstream_tiff_paths:
+                (download_dir / tp).parent.mkdir(exist_ok=True, parents=True)
+                # Write TIFF to the specified file path
+                stream_response = requests.get(
+                    f"{self.app._environment.url.geturl()}/visits/{event.button.label}/{self.app._environment.murfey_session}/upstream_tiff/{tp}",
+                    stream=True,
+                )
+                # Write the file chunk-by-chunk to avoid hogging memory
+                with open(download_dir / tp, "wb") as utiff:
+                    for chunk in stream_response.iter_content(chunk_size=32 * 1024**2):
+                        utiff.write(chunk)
+        self.app.pop_screen()
+
 
 class GainReference(Screen):
     def __init__(self, gain_reference: Path, switch_status: bool, *args, **kwargs):
@@ -773,21 +895,25 @@ class GainReference(Screen):
             if event.button.id == "suggested-gain-ref":
                 self._dir_tree._gain_reference = self._gain_reference
             visit_path = f"data/{datetime.now().year}/{self.app._environment.visit}"
-            cmd = [
+            # Set up rsync command
+            rsync_cmd = [
                 "rsync",
-                str(self._dir_tree._gain_reference),
+                f"{posix_path(self._dir_tree._gain_reference)!r}",
                 f"{self.app._environment.url.hostname}::{visit_path}/processing/{secure_filename(self._dir_tree._gain_reference.name)}",
             ]
+            # Encase in bash shell
+            cmd = ["bash", "-c", " ".join(rsync_cmd)]
             if self.app._environment.demo:
                 log.info(f"Would perform {' '.join(cmd)}")
             else:
-                gain_rsync = procrunner.run(cmd)
+                # Run rsync subprocess
+                gain_rsync = subprocess.run(cmd)
                 if gain_rsync.returncode:
                     log.warning(
-                        f"Gain reference file {self._dir_tree._gain_reference} was not successfully transferred to {visit_path}/processing"
+                        f"Gain reference file {posix_path(self._dir_tree._gain_reference)!r} was not successfully transferred to {visit_path}/processing"
                     )
             process_gain_response = requests.post(
-                url=f"{str(self.app._environment.url.geturl())}/visits/{self.app._environment.visit}/process_gain",
+                url=f"{str(self.app._environment.url.geturl())}/sessions/{self.app._environment.murfey_session}/process_gain",
                 json={
                     "gain_ref": str(self._dir_tree._gain_reference),
                     "eer": bool(
@@ -803,9 +929,9 @@ class GainReference(Screen):
                 log.info(
                     f"Gain reference file {process_gain_response.json().get('gain_ref')}"
                 )
-                self.app._environment.data_collection_parameters[
-                    "gain_ref"
-                ] = process_gain_response.json().get("gain_ref")
+                self.app._environment.data_collection_parameters["gain_ref"] = (
+                    process_gain_response.json().get("gain_ref")
+                )
                 self.app._environment.data_collection_parameters[
                     "gain_ref_superres"
                 ] = process_gain_response.json().get("gain_ref_superres")
@@ -831,8 +957,10 @@ class DirectorySelection(SwitchSelection):
         visit_dir = Path(str(event.button.label)) / self.app._visit
         visit_dir.mkdir(exist_ok=True)
         self.app._set_default_acquisition_directories(visit_dir)
-        machine_config = get_machine_config(
-            str(self.app._environment.url.geturl()), demo=self.app._environment.demo
+        machine_config = get_machine_config_client(
+            str(self.app._environment.url.geturl()),
+            instrument_name=self.app._environment.instrument_name,
+            demo=self.app._environment.demo,
         )
         for dir in machine_config["create_directories"].values():
             (visit_dir / dir).mkdir(exist_ok=True)
@@ -871,7 +999,10 @@ class DestinationSelect(Screen):
             )
             yield RadioButton("Tomography", value=self._context is TomographyContext)
         if self.app._multigrid:
-            machine_config = get_machine_config(str(self.app._environment.url.geturl()))
+            machine_config = get_machine_config_client(
+                str(self.app._environment.url.geturl()),
+                instrument_name=self.app._environment.instrument_name,
+            )
             destinations = []
             if self._destination_overrides:
                 for k, v in self._destination_overrides.items():
@@ -893,7 +1024,7 @@ class DestinationSelect(Screen):
                             not in machine_config["create_directories"].values()
                         ):
                             machine_data = requests.get(
-                                f"{self.app._environment.url.geturl()}/machine/"
+                                f"{self.app._environment.url.geturl()}/machine"
                             ).json()
                             dest = determine_default_destination(
                                 self.app._visit,
@@ -928,7 +1059,10 @@ class DestinationSelect(Screen):
                                 )
                             )
         else:
-            machine_config = get_machine_config(str(self.app._environment.url.geturl()))
+            machine_config = get_machine_config_client(
+                str(self.app._environment.url.geturl()),
+                instrument_name=self.app._environment.instrument_name,
+            )
             for s, d in self._transfer_routes.items():
                 if Path(d).name not in machine_config["create_directories"].values():
                     bulk.append(Label(f"Copy the source {s} to:"))
@@ -956,8 +1090,10 @@ class DestinationSelect(Screen):
                     i = Input(value=val, id=k.name, classes="input-destination")
                 params_bulk.append(i)
                 self._inputs[i] = k.name
-            machine_config = get_machine_config(
-                str(self.app._environment.url.geturl()), demo=self.app._environment.demo
+            machine_config = get_machine_config_client(
+                str(self.app._environment.url.geturl()),
+                instrument_name=self.app._environment.instrument_name,
+                demo=self.app._environment.demo,
             )
             if machine_config.get("superres"):
                 params_bulk.append(
@@ -1006,8 +1142,10 @@ class DestinationSelect(Screen):
 
     def on_radio_set_changed(self, event: RadioSet.Changed) -> None:
         if event.index == 0:
-            cfg = get_machine_config(
-                str(self.app._environment.url.geturl()), demo=self.app._environment.demo
+            cfg = get_machine_config_client(
+                str(self.app._environment.url.geturl()),
+                instrument_name=self.app._environment.instrument_name,
+                demo=self.app._environment.demo,
             )
             if cfg.get("modular_spa"):
                 self._context = SPAContext
@@ -1063,16 +1201,6 @@ class DestinationSelect(Screen):
                 self.app._start_rsyncer(s, d)
         for k, v in self._user_params.items():
             self.app._environment.data_collection_parameters[k] = v
-        if len(self._transfer_routes) > 1:
-            capture_post(
-                f"{self.app._environment.url.geturl()}/visits/{self.app._environment.visit}/write_connections_file",
-                json={
-                    "filename": f"murfey-{datetime.now().strftime('%Y-%m-%d-%H_%M_%S')}.txt",
-                    "destinations": [
-                        Path(n).name for n in self._transfer_routes.values()
-                    ],
-                },
-            )
         self.app.pop_screen()
         self.app.push_screen("main")
 
@@ -1129,13 +1257,12 @@ class WaitingScreen(Screen):
         self.query_one(ProgressBar).advance(1)
         if self.query_one(ProgressBar).progress == self.query_one(ProgressBar).total:
             requests.post(
-                f"{self.app._environment.url.geturl()}/clients/{self.app._environment.client_id}/successful_processing"
+                f"{self.app._environment.url.geturl()}/sessions/{self.app._environment.murfey_session}/successful_processing"
             )
             requests.delete(
-                f"{self.app._environment.url.geturl()}/clients/{self.app._environment.client_id}/session"
+                f"{self.app._environment.url.geturl()}/instruments/{self._environment.instrument_name}/clients/{self.app._environment.client_id}/session"
             )
             self.app.exit()
-            exit()
 
 
 class MainScreen(Screen):
