@@ -1,32 +1,83 @@
+"""
+Contains classes that are used to store information on the metadata and status of jobs
+of the sessions that Murfey is overseeing, along with the relationships between them.
+"""
+
 from typing import List, Optional
 
 import sqlalchemy
 from sqlmodel import Field, Relationship, SQLModel, create_engine
 
+"""
+GENERAL
+"""
+
+
+class MurfeyUser(SQLModel, table=True):  # type: ignore
+    username: str = Field(primary_key=True)
+    hashed_password: str
+
+
+class MagnificationLookup(SQLModel, table=True):  # type: ignore
+    magnification: int = Field(primary_key=True)
+    pixel_size: float = Field(primary_key=True)
+
 
 class ClientEnvironment(SQLModel, table=True):  # type: ignore
     client_id: Optional[int] = Field(primary_key=True, unique=True)
     visit: str = Field(default="")
     session_id: Optional[int] = Field(foreign_key="session.id")
     connected: bool
-    rsync_instances: List["RsyncInstance"] = Relationship(
-        back_populates="client", sa_relationship_kwargs={"cascade": "delete"}
-    )
 
 
 class RsyncInstance(SQLModel, table=True):  # type: ignore
     source: str = Field(primary_key=True)
     destination: str = Field(primary_key=True, default="")
-    client_id: int = Field(foreign_key="clientenvironment.client_id", primary_key=True)
+    session_id: int = Field(foreign_key="session.id", primary_key=True)
+    tag: str = Field(default="")
     files_transferred: int = Field(default=0)
     files_counted: int = Field(default=0)
     transferring: bool = Field(default=False)
-    client: Optional[ClientEnvironment] = Relationship(back_populates="rsync_instances")
+    session: Optional["Session"] = Relationship(back_populates="rsync_instances")
 
 
 class Session(SQLModel, table=True):  # type: ignore
     id: int = Field(primary_key=True)
     name: str
+    visit: str = Field(default="")
+    started: bool = Field(default=False)
+    current_gain_ref: str = Field(default="")
+    instrument_name: str = Field(default="")
+
+    # CLEM Workflow
+
+    # LIF files collected, if any
+    lif_files: List["CLEMLIFFile"] = Relationship(
+        back_populates="session",
+        sa_relationship_kwargs={"cascade": "delete"},
+    )
+    # TIFF files collected, if any
+    tiff_files: List["CLEMTIFFFile"] = Relationship(
+        back_populates="session",
+        sa_relationship_kwargs={"cascade": "delete"},
+    )
+    # Metadata files generated
+    metadata_files: List["CLEMImageMetadata"] = Relationship(
+        back_populates="session", sa_relationship_kwargs={"cascade": "delete"}
+    )
+    # Image series associated with this session
+    image_series: List["CLEMImageSeries"] = Relationship(
+        back_populates="session",
+        sa_relationship_kwargs={"cascade": "delete"},
+    )
+    # Image stacks associated with this session
+    image_stacks: List["CLEMImageStack"] = Relationship(
+        back_populates="session",
+        sa_relationship_kwargs={"cascade": "delete"},
+    )
+
+    # TEM Workflow
+
     tilt_series: List["TiltSeries"] = Relationship(
         back_populates="session", sa_relationship_kwargs={"cascade": "delete"}
     )
@@ -42,6 +93,237 @@ class Session(SQLModel, table=True):  # type: ignore
     foil_holes: List["FoilHole"] = Relationship(
         back_populates="session", sa_relationship_kwargs={"cascade": "delete"}
     )
+    rsync_instances: List[RsyncInstance] = Relationship(
+        back_populates="session", sa_relationship_kwargs={"cascade": "delete"}
+    )
+
+
+"""
+CLEM WORKFLOW
+"""
+
+
+class CLEMLIFFile(SQLModel, table=True):  # type: ignore
+    """
+    Database recording the different LIF files acquired during the CLEM session, as
+    well as the different image series stored within them.
+    """
+
+    id: Optional[int] = Field(default=None, primary_key=True)
+    file_path: str = Field(index=True)  # Path to LIF file
+
+    # The CLEM session this series belongs to
+    session: Optional["Session"] = Relationship(
+        back_populates="lif_files"
+    )  # Many to one
+    session_id: Optional[int] = Field(
+        foreign_key="session.id",
+        default=None,
+    )
+
+    master_metadata: Optional[str] = Field(
+        index=True
+    )  # Path to master metadata generated from LIF file
+
+    # Offspring
+    child_metadata: List["CLEMImageMetadata"] = Relationship(
+        back_populates="parent_lif",
+        sa_relationship_kwargs={"cascade": "delete"},
+    )  # One to many
+    child_series: List["CLEMImageSeries"] = Relationship(
+        back_populates="parent_lif",
+        sa_relationship_kwargs={"cascade": "delete"},
+    )  # One to many
+    child_stacks: List["CLEMImageStack"] = Relationship(
+        back_populates="parent_lif",
+        sa_relationship_kwargs={"cascade": "delete"},
+    )  # One to many
+
+
+class CLEMTIFFFile(SQLModel, table=True):  # type: ignore
+    """
+    Database to record each raw TIFF file acquired during a CLEM session, which are
+    used to create an image stack
+    """
+
+    id: Optional[int] = Field(default=None, primary_key=True)
+    file_path: str = Field(index=True)  # File path to TIFF file on system
+
+    session: Optional["Session"] = Relationship(
+        back_populates="tiff_files"
+    )  # Many to one
+    session_id: Optional[int] = Field(
+        foreign_key="session.id",
+        default=None,
+    )
+
+    # Metadata associated with this TIFF file
+    associated_metadata: Optional["CLEMImageMetadata"] = Relationship(
+        back_populates="associated_tiffs",
+    )  # Many to one
+    metadata_id: Optional[int] = Field(
+        foreign_key="clemimagemetadata.id",
+        default=None,
+    )
+
+    # Image series it contributes to
+    child_series: Optional["CLEMImageSeries"] = Relationship(
+        back_populates="parent_tiffs"
+    )  # Many to one
+    series_id: Optional[int] = Field(
+        foreign_key="clemimageseries.id",
+        default=None,
+    )
+
+    # Image stack it contributes to
+    child_stack: Optional["CLEMImageStack"] = Relationship(
+        back_populates="parent_tiffs"
+    )  # Many to one
+    stack_id: Optional[int] = Field(
+        foreign_key="clemimagestack.id",
+        default=None,
+    )
+
+
+class CLEMImageMetadata(SQLModel, table=True):  # type: ignore
+
+    id: Optional[int] = Field(default=None, primary_key=True)
+    file_path: str = Field(index=True)  # Full path to metadata file
+
+    session: Optional["Session"] = Relationship(back_populates="metadata_files")
+    session_id: Optional[int] = Field(foreign_key="session.id")  # Many to one
+
+    # The parent LIF file this metadata originates from, if any
+    parent_lif: Optional[CLEMLIFFile] = Relationship(
+        back_populates="child_metadata",
+    )  # Many to one
+    parent_lif_id: Optional[int] = Field(
+        foreign_key="clemliffile.id",
+        default=None,
+    )
+    # The TIFF files related to this file
+    associated_tiffs: List["CLEMTIFFFile"] = Relationship(
+        back_populates="associated_metadata",
+        sa_relationship_kwargs={"cascade": "delete"},
+    )  # One to many
+
+    # Associated series
+    associated_series: Optional["CLEMImageSeries"] = Relationship(
+        back_populates="associated_metadata",
+        sa_relationship_kwargs={"cascade": "delete"},
+    )  # One to one
+
+    # Associated image stacks
+    associated_stacks: List["CLEMImageStack"] = Relationship(
+        back_populates="associated_metadata",
+        sa_relationship_kwargs={"cascade": "delete"},
+    )  # One to many
+
+
+class CLEMImageSeries(SQLModel, table=True):  # type: ignore
+    """
+    Database recording the individual files associated with a series, which are to be
+    processed together as a group. These files could stem from a parent LIF file, or
+    have been compiled together from individual TIFF files.
+    """
+
+    id: Optional[int] = Field(default=None, primary_key=True)
+    series_name: str = Field(
+        index=True
+    )  # Name of the series, as determined from the metadata
+
+    session: Optional["Session"] = Relationship(
+        back_populates="image_series"
+    )  # Many to one
+    session_id: Optional[int] = Field(
+        foreign_key="session.id", default=None, unique=False
+    )
+
+    # The parent LIF file this series originates from, if any
+    parent_lif: Optional["CLEMLIFFile"] = Relationship(
+        back_populates="child_series",
+    )  # Many to one
+    parent_lif_id: Optional[int] = Field(
+        foreign_key="clemliffile.id",
+        default=None,
+    )
+
+    # The parent TIFF files used to build up the image stacks in the series, if any
+    parent_tiffs: List["CLEMTIFFFile"] = Relationship(
+        back_populates="child_series", sa_relationship_kwargs={"cascade": "delete"}
+    )  # One to many
+
+    # Metadata file for this series
+    associated_metadata: Optional["CLEMImageMetadata"] = Relationship(
+        back_populates="associated_series",
+    )  # One to one
+    metadata_id: Optional[int] = Field(
+        foreign_key="clemimagemetadata.id",
+        default=None,
+    )
+
+    # Databases of the image stacks that comprise this series
+    child_stacks: List["CLEMImageStack"] = Relationship(
+        back_populates="parent_series",
+        sa_relationship_kwargs={"cascade": "delete"},
+    )  # One to many
+
+    # Process checklist for series
+    number_of_members: int = (
+        0  # Expected number of image stacks belonging to this series
+    )
+    composite_created: bool = False  # Has a composite image been created?
+    composite_image: Optional[str] = None  # Full path to composite image
+
+
+class CLEMImageStack(SQLModel, table=True):  # type: ignore
+    """
+    Database to keep track of the processing status of a single image stack.
+    """
+
+    id: Optional[int] = Field(default=None, primary_key=True)
+    file_path: str = Field(index=True)  # Full path to the file
+    channel_name: Optional[str] = None  # Color associated with stack
+
+    session: Optional["Session"] = Relationship(
+        back_populates="image_stacks"
+    )  # Many to one
+    session_id: Optional[int] = Field(foreign_key="session.id")
+
+    # LIF file this stack originated from
+    parent_lif: Optional["CLEMLIFFile"] = Relationship(
+        back_populates="child_stacks",
+    )  # Many to one
+    parent_lif_id: Optional[int] = Field(foreign_key="clemliffile.id", default=None)
+
+    # TIFF files used to build this stack
+    parent_tiffs: List["CLEMTIFFFile"] = Relationship(
+        back_populates="child_stack",
+        sa_relationship_kwargs={"cascade": "delete"},
+    )  # One to many
+
+    # Metadata associated with statck
+    associated_metadata: Optional["CLEMImageMetadata"] = Relationship(
+        back_populates="associated_stacks",
+    )  # Many to one
+    metadata_id: Optional[int] = Field(
+        foreign_key="clemimagemetadata.id",
+        default=None,
+    )
+
+    # Image series this image stack belongs to
+    parent_series: Optional["CLEMImageSeries"] = Relationship(
+        back_populates="child_stacks",
+    )  # Many to one
+    series_id: Optional[int] = Field(
+        foreign_key="clemimageseries.id",
+        default=None,
+    )
+
+
+"""
+TEM SESSION AND PROCESSING WORKFLOW
+"""
 
 
 class TiltSeries(SQLModel, table=True):  # type: ignore
@@ -49,7 +331,7 @@ class TiltSeries(SQLModel, table=True):  # type: ignore
     tag: str
     rsync_source: str
     session_id: int = Field(foreign_key="session.id")
-    complete: bool = False
+    tilt_series_length: int = -1
     processing_requested: bool = False
     session: Optional[Session] = Relationship(back_populates="tilt_series")
     tilts: List["Tilt"] = Relationship(
@@ -69,6 +351,8 @@ class DataCollectionGroup(SQLModel, table=True):  # type: ignore
     id: int = Field(primary_key=True, unique=True)
     session_id: int = Field(foreign_key="session.id", primary_key=True)
     tag: str = Field(primary_key=True)
+    atlas_id: Optional[int] = None
+    atlas_pixel_size: Optional[float] = None
     atlas: str = ""
     sample: Optional[int] = None
     session: Optional[Session] = Relationship(back_populates="data_collection_groups")
@@ -76,11 +360,11 @@ class DataCollectionGroup(SQLModel, table=True):  # type: ignore
         back_populates="data_collection_group",
         sa_relationship_kwargs={"cascade": "delete"},
     )
-    tomography_preprocessing_parameters: List[
-        "TomographyPreprocessingParameters"
-    ] = Relationship(
-        back_populates="data_collection_group",
-        sa_relationship_kwargs={"cascade": "delete"},
+    tomography_preprocessing_parameters: List["TomographyPreprocessingParameters"] = (
+        Relationship(
+            back_populates="data_collection_group",
+            sa_relationship_kwargs={"cascade": "delete"},
+        )
     )
 
 
@@ -119,10 +403,11 @@ class ProcessingJob(SQLModel, table=True):  # type: ignore
     spa_feedback_parameters: List["SPAFeedbackParameters"] = Relationship(
         back_populates="processing_job", sa_relationship_kwargs={"cascade": "delete"}
     )
-    tomography_processing_parameters: List[
-        "TomographyProcessingParameters"
-    ] = Relationship(
-        back_populates="processing_job", sa_relationship_kwargs={"cascade": "delete"}
+    tomography_processing_parameters: List["TomographyProcessingParameters"] = (
+        Relationship(
+            back_populates="processing_job",
+            sa_relationship_kwargs={"cascade": "delete"},
+        )
     )
     ctf_parameters: List["CtfParameters"] = Relationship(
         back_populates="processing_job", sa_relationship_kwargs={"cascade": "delete"}
@@ -415,12 +700,13 @@ class Class3D(SQLModel, table=True):  # type: ignore
 
 
 class RefineParameters(SQLModel, table=True):  # type: ignore
+    tag: str = Field(primary_key=True)
     refine_dir: str = Field(primary_key=True)
     pj_id: int = Field(primary_key=True, foreign_key="processingjob.id")
     murfey_id: int = Field(foreign_key="murfeyledger.id")
     class3d_dir: str
     class_number: int
-    run: bool = True
+    run: bool = False
     processing_job: Optional[ProcessingJob] = Relationship(
         back_populates="refine_parameters"
     )
@@ -430,6 +716,7 @@ class RefineParameters(SQLModel, table=True):  # type: ignore
 
 
 class Refine3D(SQLModel, table=True):  # type: ignore
+    tag: str = Field(primary_key=True)
     refine_dir: str = Field(primary_key=True)
     pj_id: int = Field(primary_key=True, foreign_key="processingjob.id")
     murfey_id: int = Field(foreign_key="murfeyledger.id")
@@ -456,6 +743,11 @@ class BFactors(SQLModel, table=True):  # type: ignore
     resolution: float
 
 
+"""
+FUNCTIONS
+"""
+
+
 def setup(url: str):
     engine = create_engine(url)
     SQLModel.metadata.create_all(engine)
@@ -463,7 +755,7 @@ def setup(url: str):
 
 def clear(url: str):
     engine = create_engine(url)
-    metadata = sqlalchemy.MetaData(engine)
-    metadata.reflect()
-
+    metadata = sqlalchemy.MetaData()
+    metadata.create_all(engine)
+    metadata.reflect(engine)
     metadata.drop_all(engine)
