@@ -1,7 +1,10 @@
 from __future__ import annotations
 
 import os
-from typing import BinaryIO
+from pathlib import Path
+from typing import BinaryIO, Literal
+
+from murfey.util import secure_path
 
 
 def _count_ifds(file_stream: BinaryIO) -> int:
@@ -9,7 +12,7 @@ def _count_ifds(file_stream: BinaryIO) -> int:
     # get the byte order mark
     # for TIFFs II means intel ordering (little-endian) and MM means motorola ordering (big-endian)
     bom = bytes(file_stream.read(2)).decode("utf-8")
-    byteorder = "little" if bom == "II" else "big"
+    byteorder: Literal["little", "big"] = "little" if bom == "II" else "big"
     file_stream.seek(8, 0)
     ifd = int.from_bytes(file_stream.read(8), byteorder)
     n = 0
@@ -25,6 +28,6 @@ def _count_ifds(file_stream: BinaryIO) -> int:
 
 
 def num_frames(eer_path: os.PathLike) -> int:
-    with open(eer_path, "rb") as eer:
+    with open(secure_path(Path(eer_path)), "rb") as eer:
         n = _count_ifds(eer)
     return n
