@@ -6,6 +6,12 @@ from typing import Any, Dict, List, Optional
 
 from pydantic import BaseModel
 
+"""
+General Models
+==============
+Models used in multiple workflows.
+"""
+
 
 class Visit(BaseModel):
     start: datetime
@@ -28,136 +34,6 @@ class Visit(BaseModel):
         )
 
 
-class Sample(BaseModel):
-    sample_group_id: int
-    sample_id: int
-    subsample_id: int
-    image_path: Optional[Path]
-
-
-class BLSampleImageParameters(BaseModel):
-    sample_id: int
-    sample_path: Path
-
-
-class BLSampleParameters(BaseModel):
-    sample_group_id: int
-
-
-class BLSubSampleParameters(BaseModel):
-    sample_id: int
-    image_path: Optional[Path] = None
-
-
-class ContextInfo(BaseModel):
-    experiment_type: str
-    acquisition_software: str
-
-
-class ClientInfo(BaseModel):
-    id: int
-
-
-class RsyncerInfo(BaseModel):
-    source: str
-    destination: str
-    client_id: int
-    transferring: bool = True
-    increment_count: int = 1
-    bytes: int = 0
-    increment_data_count: int = 0
-    data_bytes: int = 0
-
-
-class ClearanceKeys(BaseModel):
-    data_collection_group: List[str]
-    data_collection: List[str]
-    processing_job: List[str]
-    autoproc_program: List[str]
-
-
-class File(BaseModel):
-    name: str
-    description: str
-    size: int
-    timestamp: float
-
-
-class SPAProcessingParameters(BaseModel):
-    job_id: int
-
-
-class ProcessFile(BaseModel):
-    path: str
-    description: str
-    tag: str
-    data_collection_id: Optional[int]
-    image_number: int
-    pixel_size: float
-    dose_per_frame: float
-    processing_job: Optional[int] = None
-    autoproc_program_id: Optional[int] = None
-    mc_uuid: Optional[int] = None
-    voltage: float = 300
-    mc_binning: int = 1
-    gain_ref: Optional[str] = None
-    extract_downscale: int = 1
-    eer_fractionation_file: Optional[str] = None
-    group_tag: Optional[str] = None
-
-
-class SPAProcessFile(BaseModel):
-    tag: str
-    path: str
-    description: str
-    processing_job: Optional[int]
-    data_collection_id: Optional[int]
-    image_number: int
-    autoproc_program_id: Optional[int]
-    foil_hole_id: int
-    pixel_size: Optional[float]
-    dose_per_frame: Optional[float]
-    mc_binning: Optional[int] = 1
-    gain_ref: Optional[str] = None
-    extract_downscale: bool = True
-    eer_fractionation_file: Optional[str] = None
-    source: str = ""
-
-
-class TiltInfo(BaseModel):
-    tilt_series_tag: str
-    movie_path: str
-    source: str
-
-
-class TiltSeriesInfo(BaseModel):
-    client_id: int
-    tag: str
-    source: str
-
-
-class TiltSeriesGroupInfo(BaseModel):
-    tags: List[str]
-    source: str
-
-
-class CompletedTiltSeries(BaseModel):
-    tilt_series: List[str]
-    rsync_source: str
-
-
-class TiltSeriesProcessingDetails(BaseModel):
-    name: str
-    file_tilt_list: str
-    dcid: int
-    processing_job: int
-    autoproc_program_id: int
-    motion_corrected_path: str
-    movie_id: int
-    pixel_size: float
-    manual_tilt_offset: int
-
-
 class SuggestedPathParameters(BaseModel):
     base_path: Path
     touch: bool = False
@@ -165,11 +41,13 @@ class SuggestedPathParameters(BaseModel):
 
 
 class DCGroupParameters(BaseModel):
+    # DC = Data collection
     experiment_type: str
     experiment_type_id: int
     tag: str
     atlas: str = ""
     sample: Optional[int] = None
+    atlas_pixel_size: int = 0
 
 
 class DCParameters(BaseModel):
@@ -192,31 +70,146 @@ class DCParameters(BaseModel):
     data_collection_tag: str = ""
 
 
-class PreprocessingParametersTomo(BaseModel):
-    dose_per_frame: float
-    gain_ref: Optional[str]
+class ProcessingJobParameters(BaseModel):
+    tag: str
+    source: str
+    recipe: str
+    parameters: Dict[str, Any] = {}
+    experiment_type: str = "spa"
+
+
+class RegistrationMessage(BaseModel):
+    registration: str
+    params: Optional[Dict[str, Any]] = None
+
+
+class File(BaseModel):
+    name: str
+    description: str
+    size: int
+    timestamp: datetime
+    full_path: str
+
+
+class ConnectionFileParameters(BaseModel):
+    filename: str
+    destinations: List[str]
+
+
+class SessionInfo(BaseModel):
+    session_id: Optional[int]
+    session_name: str = ""
+    rescale: bool = True
+
+
+class ContextInfo(BaseModel):
     experiment_type: str
-    voltage: float
-    image_size_x: int
-    image_size_y: int
-    pixel_size_on_image: str
-    motion_corr_binning: int
-    manual_tilt_offset: float
-    file_extension: str
+    acquisition_software: str
+
+
+class ClientInfo(BaseModel):
+    id: int
+
+
+class RsyncerSource(BaseModel):
+    source: str
+
+
+class RsyncerInfo(BaseModel):
+    source: str
+    destination: str
+    session_id: int
+    transferring: bool = True
+    increment_count: int = 1
+    bytes: int = 0
+    increment_data_count: int = 0
+    data_bytes: int = 0
+    tag: str = ""
+
+
+class ClearanceKeys(BaseModel):
+    data_collection_group: List[str]
+    data_collection: List[str]
+    processing_job: List[str]
+    autoproc_program: List[str]
+
+
+class GainReference(BaseModel):
+    gain_ref: Path
+    rescale: bool = True
+    eer: bool = False
+    tag: str = ""
+
+
+class FractionationParameters(BaseModel):
+    fractionation: int
+    dose_per_frame: float
+    num_frames: int = 0
+    eer_path: Optional[str] = None
+    fractionation_file_name: str = "eer_fractionation.txt"
+
+
+"""
+FIB
+===
+Models related to FIB, as part of correlative workflow with TEM.
+"""
+
+
+class Sample(BaseModel):
+    sample_group_id: int
+    sample_id: int
+    subsample_id: int
+    image_path: Optional[Path]
+
+
+class BLSampleImageParameters(BaseModel):
+    sample_id: int
+    sample_path: Path
+
+
+class BLSampleParameters(BaseModel):
+    sample_group_id: int
+
+
+class BLSubSampleParameters(BaseModel):
+    sample_id: int
+    image_path: Optional[Path] = None
+
+
+class MillingParameters(BaseModel):
+    lamella_number: int
+    images: List[str]
+    raw_directory: str
+
+
+"""
+Single Particle Analysis
+========================
+Models related to the single-particle analysis workflow.
+"""
+
+
+class SPAProcessingParameters(BaseModel):
+    job_id: int
+
+
+class SPAProcessFile(BaseModel):
     tag: str
-    tilt_series_tag: str
-    eer_fractionation_file: Optional[str]
-
-    class Base(BaseModel):
-        dose_per_frame: float
-        gain_ref: Optional[str]
-        manual_tilt_offset: float
-
-
-class ProcessingParametersTomo(BaseModel):
-    manual_tilt_offset: int
-    tag: str
-    tilt_series_tag: str
+    path: str
+    description: str
+    processing_job: Optional[int]
+    data_collection_id: Optional[int]
+    image_number: int
+    autoproc_program_id: Optional[int]
+    foil_hole_id: Optional[int]
+    pixel_size: Optional[float]
+    dose_per_frame: Optional[float]
+    mc_binning: Optional[int] = 1
+    gain_ref: Optional[str] = None
+    extract_downscale: bool = True
+    eer_fractionation_file: Optional[str] = None
+    source: str = ""
 
 
 class ProcessingParametersSPA(BaseModel):
@@ -258,49 +251,6 @@ class ProcessingParametersSPA(BaseModel):
         eer_fractionation: int
 
 
-class ProcessingJobParameters(BaseModel):
-    tag: str
-    recipe: str
-    parameters: Dict[str, Any] = {}
-    experiment_type: str = "spa"
-
-
-class RegistrationMessage(BaseModel):
-    registration: str
-    params: Optional[Dict[str, Any]] = None
-
-
-class ConnectionFileParameters(BaseModel):
-    filename: str
-    destinations: List[str]
-
-
-class GainReference(BaseModel):
-    gain_ref: Path
-    rescale: bool = True
-    eer: bool = False
-
-
-class SessionInfo(BaseModel):
-    session_id: Optional[int]
-    session_name: str = ""
-    rescale: bool = True
-
-
-class MillingParameters(BaseModel):
-    lamella_number: int
-    images: List[str]
-    raw_directory: str
-
-
-class FractionationParameters(BaseModel):
-    fractionation: int
-    dose_per_frame: float
-    num_frames: int = 0
-    eer_path: Optional[str] = None
-    fractionation_file_name: str = "eer_fractionation.txt"
-
-
 class GridSquareParameters(BaseModel):
     tag: str
     x_location: Optional[float] = None
@@ -311,8 +261,11 @@ class GridSquareParameters(BaseModel):
     readout_area_y: Optional[int] = None
     thumbnail_size_x: Optional[int] = None
     thumbnail_size_y: Optional[int] = None
+    height: Optional[int] = None
+    width: Optional[int] = None
     pixel_size: Optional[float] = None
     image: str = ""
+    angle: Optional[float] = None
 
 
 class FoilHoleParameters(BaseModel):
@@ -328,3 +281,103 @@ class FoilHoleParameters(BaseModel):
     thumbnail_size_y: Optional[int] = None
     pixel_size: Optional[float] = None
     image: str = ""
+    diameter: Optional[float] = None
+
+
+class PostInfo(BaseModel):
+    url: str
+    data: dict
+
+
+class MultigridWatcherSetup(BaseModel):
+    source: Path
+    skip_existing_processing: bool = False
+    destination_overrides: Dict[Path, str] = {}
+    rsync_restarts: List[str] = []
+
+
+class CurrentGainRef(BaseModel):
+    path: str
+
+
+class Token(BaseModel):
+    access_token: str
+    token_type: str
+
+
+"""
+Tomography
+==========
+Models related to the tomographic reconstruction workflow.
+"""
+
+
+class ProcessFile(BaseModel):  # Rename to TomoProcessFile
+    path: str
+    description: str
+    tag: str
+    data_collection_id: Optional[int]
+    image_number: int
+    pixel_size: float
+    dose_per_frame: float
+    processing_job: Optional[int] = None
+    autoproc_program_id: Optional[int] = None
+    mc_uuid: Optional[int] = None
+    voltage: float = 300
+    mc_binning: int = 1
+    gain_ref: Optional[str] = None
+    extract_downscale: int = 1
+    eer_fractionation_file: Optional[str] = None
+    group_tag: Optional[str] = None
+
+
+class TiltInfo(BaseModel):
+    tilt_series_tag: str
+    movie_path: str
+    source: str
+
+
+class TiltSeriesInfo(BaseModel):
+    session_id: int
+    tag: str
+    source: str
+
+
+class TiltSeriesGroupInfo(BaseModel):
+    tags: List[str]
+    source: str
+    tilt_series_lengths: List[int]
+
+
+class CompletedTiltSeries(BaseModel):
+    tilt_series: List[str]
+    rsync_source: str
+
+
+class PreprocessingParametersTomo(BaseModel):
+    dose_per_frame: float
+    gain_ref: Optional[str]
+    experiment_type: str
+    voltage: float
+    image_size_x: int
+    image_size_y: int
+    pixel_size_on_image: str
+    motion_corr_binning: int
+    manual_tilt_offset: float
+    file_extension: str
+    tag: str
+    tilt_series_tag: str
+    eer_fractionation_file: Optional[str]
+    eer_fractionation: int
+
+    class Base(BaseModel):
+        dose_per_frame: float
+        gain_ref: Optional[str]
+        manual_tilt_offset: float
+        eer_fractionation: int
+
+
+class ProcessingParametersTomo(BaseModel):
+    manual_tilt_offset: int
+    tag: str
+    tilt_series_tag: str
